#include "stdafx.h"
#include "BaseTracker.h"
#include "BaseTrackerDlg.h"

#include <direct.h>     // For _getcwd()

#include <iostream>
#include <fstream>
#include <cstdlib>
using namespace std;

#include "AVIViewer.h"
#include "EdgeAppearanceModel.h"
#include "PatchAppearanceModel.h"
#include "EdgePixelGrabber.h"
#include "EdgeThresholdSetter.h"
#include "ImgFun.h"
#include "KinematicTree.h"
#include "Matrix.h"
#include "ModelPoint.h"
#include "Registration.h"
#include "StateSetter.h"
#include "TrackerException.h"
#include "MathFun.h"
#include "Vector.h"

#include "cv.h"
#include "highgui.h"

#include "InitEdges.h"
#include "EdgeGrabberDlg.h"
#include "SetStateDlg.h"
#include "TrackingDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

/** @defgroup mfc_group MFC Stuff 
    @ingroup gui_group */
/* @{ */
class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};
/* @} */

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBaseTrackerDlg dialog

CBaseTrackerDlg::CBaseTrackerDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBaseTrackerDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CBaseTrackerDlg)
	m_init_appearance_frame = 0;
	m_set_state_frame = 0;
	m_opf_nsamples = 0;
	m_method = UnscentedPF;
	m_mcstep = Yes;
	m_opti = Yes;
	m_neff = No;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CBaseTrackerDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBaseTrackerDlg)
	DDX_Text(pDX, IDC_APPEARANCE_FRAME, m_init_appearance_frame);
	DDX_Text(pDX, IDC_STATE_FRAME, m_set_state_frame);
	DDX_Text(pDX, IDC_OPF_NSAMPLES, m_opf_nsamples);
	DDX_CBIndex(pDX, IDC_COMBO_METHODS, m_method);
	DDX_CBIndex(pDX, IDC_COMBO_MCMOVE, m_mcstep);
	DDX_CBIndex(pDX, IDC_COMBO_OPTI, m_opti);
	DDX_CBIndex(pDX, IDC_COMBO_Neff, m_neff);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CBaseTrackerDlg, CDialog)
	//{{AFX_MSG_MAP(CBaseTrackerDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_LOAD_KINEMATICS, OnLoadKinematics)
	ON_BN_CLICKED(IDC_LOAD_AVI, OnLoadAvi)
	ON_BN_CLICKED(IDC_SET_THRESHOLDS, OnSetThresholds)
	ON_BN_CLICKED(IDC_INIT_APPEARANCE, OnInitAppearance)
	ON_BN_CLICKED(IDC_SET_STATE, OnSetState)
	ON_BN_CLICKED(IDC_TRACK, OnTrack)
	ON_BN_CLICKED(IDC_SAVE_PROJECT, OnSaveProject)
	ON_BN_CLICKED(IDC_LOAD_PROJECT, OnLoadProject)
	ON_BN_CLICKED(IDC_LOAD_DYNAMICS, OnLoadDynamics)
	ON_BN_CLICKED(IDC_OptimizedPF, OnOptimizedPF)
	ON_BN_CLICKED(IDC_SAVETRACKER, OnSavetracker)
	ON_BN_CLICKED(IDC_OPF_BATCH, OnOpfBatch)
	ON_BN_CLICKED(IDC_METHOD_TRACK, OnMethodTrack)
	ON_BN_CLICKED(IDC_METHOD_BATCH, OnMethodBatch)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBaseTrackerDlg message handlers

BOOL CBaseTrackerDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
    m_init_appearance_frame = 0;
    m_set_state_frame = 0;
	m_opf_nsamples = 1;
	m_method = RegularizedPF;
	m_opti = Yes;
	m_mcstep = No;
	m_neff = No;
    UpdateData(FALSE);
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CBaseTrackerDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CBaseTrackerDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CBaseTrackerDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}



void CBaseTrackerDlg::OnLoadKinematics() 
{
    CFileDialog openDlg(true, "kt", NULL, 
        OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
        "Kinematic Tree (*.kt)|*.kt|All Files (*.*)|*.*||",
        this);
	
	// Tell the dialog to look in the working directory
    char buffer[512];
    _getcwd(buffer, 512);
    openDlg.m_ofn.lpstrInitialDir = buffer;

	if (IDOK == openDlg.DoModal()) {
        char *msg;
        if (!project.openKT(openDlg.GetPathName(), &msg)) {
            AfxMessageBox(msg);
            delete [] msg;
        }
   	}
}


void CBaseTrackerDlg::OnLoadAvi() 
{
    CFileDialog openDlg(true, "avi", NULL, 
        OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
        "AVI File (*.avi)|*.avi|All Files (*.*)|*.*||",
        this);
	
	// Tell the dialog to look in the working directory
    char buffer[512];
    _getcwd(buffer, 512);
    openDlg.m_ofn.lpstrInitialDir = buffer;

	if (IDOK == openDlg.DoModal()) {
		char *msg;
        if (!project.openAVI(openDlg.GetPathName(), &msg)) {
            AfxMessageBox(msg);
            delete [] msg;
        }
	}	
}


void CBaseTrackerDlg::OnSetThresholds() 
{
	/*if (!project.avi.isOpen()) {
		AfxMessageBox("Must open an AVI file first.");
		return;
	}

	int low, high;
	project.eam.getEdgeThresholds(low, high);

    EdgeThresholdSetter ets(project.avi.getGrayFrame(0), low, high);
    InitEdges initEdgesDlg(ets, this);
    initEdgesDlg.DoModal();
    ets.getThresholds(low, high);
    project.eam.setEdgeThresholds(low, high);
    */
}

void CBaseTrackerDlg::OnInitAppearance() 
{
	/*
    if (!project.avi.isOpen()) {
		AfxMessageBox("Must open an AVI file first.");
		return;
	}

    if (project.kt == NULL) {
        AfxMessageBox("Must open a kinematic model first.");
        return;
    }
		
	UpdateData();
	if (m_init_appearance_frame < 0 
			|| m_init_appearance_frame >= project.avi.getLength()) {
		AfxMessageBox("Invalid frame number.");
		return;
	}

	EdgePixelGrabber *epg = EdgePixelGrabber::getInstance();
    epg->initialize(project.kt, &project.eam, 
		project.avi.getGrayFrame(m_init_appearance_frame));

    EdgeGrabberDlg edgeGrabberDlg(this);
    
    if (IDOK == edgeGrabberDlg.DoModal()) {
        IplImage **linkImgs = new IplImage*[project.kt->countLinks()];
        CvPoint *origins = new CvPoint[project.kt->countLinks()];
        epg->getData(linkImgs, origins);
        project.eam.initialize(project.kt->countLinks(), linkImgs, origins, 
						       project.kt);
    }
    */
}

void CBaseTrackerDlg::OnSetState() 
{
	if (!project.avi.isOpen()) {
		AfxMessageBox("Must open an AVI file first.");
		return;
	}

    if (project.kt == NULL) {
        AfxMessageBox("Must open a kinematic model first.");
        return;
    }    
    
    if (project.dynamics == NULL) {
        AfxMessageBox("Must open a dynamic model first.");
        return;
    }
		
	UpdateData();
	if (m_set_state_frame < 0 
			|| m_set_state_frame >= project.avi.getLength()) {
		AfxMessageBox("Invalid frame number.");
		return;
	}
	
    // Get the previous state set for this frame, if any
    Vector *state = project.getInitState(m_set_state_frame);
    if (state != NULL) {
        project.kt->setState(*state);
    }

    // Display dialog to allow setting of state
    StateSetter::getInstance()->initialize(project.kt, &project.eam, 
        project.avi.getFrame(m_set_state_frame), m_set_state_frame);
    SetStateDlg setStateDlg(&project, project.kt, 
        StateSetter::getInstance(), this);
    setStateDlg.DoModal();
}

void CBaseTrackerDlg::OnLoadDynamics() 
{
    CFileDialog openDlg(true, "dyn", NULL, 
        OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
        "DYN (*.dyn)|*.dyn|All Files (*.*)|*.*||",
        this);
	
	// Tell the dialog to look in the working directory
    char buffer[512];
    _getcwd(buffer, 512);
    openDlg.m_ofn.lpstrInitialDir = buffer;

	if (IDOK == openDlg.DoModal()) {
        char *msg;
        if (!project.openDynamics(openDlg.GetPathName(), &msg)) {
            AfxMessageBox(msg);
            delete [] msg;
        }
   	}
}

void CBaseTrackerDlg::OnTrack() 
{
	char *msg;
    bool ready = project.readyToTrack(&msg);

    if (!ready) {
        AfxMessageBox(msg);
        delete [] msg;
        return;
    }
    
    TrackingDlg trackingDlg(&project, this);
    trackingDlg.DoModal();
}


void CBaseTrackerDlg::OnSaveProject() 
{
    CFileDialog saveDlg(false, "tpj", NULL, 
        OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
        "Tracking Project File (*.tpj)|*.tpj|All Files (*.*)|*.*||",
        this);

    // Tell the dialog to look in the working directory
    char buffer[512];
    _getcwd(buffer, 512);
    saveDlg.m_ofn.lpstrInitialDir = buffer;

    if (IDOK == saveDlg.DoModal()) {
        ofstream fout(saveDlg.GetPathName());
        project.writeToFile(fout);
        fout.close();
    }
}

void CBaseTrackerDlg::OnLoadProject() 
{
    CFileDialog openDlg(true, "tpj", NULL, 
        OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
        "TPJ File (*.tpj)|*.tpj|All Files (*.*)|*.*||",
        this);
	
	// Tell the dialog to look in the working directory
    char buffer[512];
    _getcwd(buffer, 512);
    openDlg.m_ofn.lpstrInitialDir = buffer;

	CString path;

	if (IDOK == openDlg.DoModal()) {
		path = openDlg.GetPathName();
        ifstream fin(path);
        char *msg;
        if (!project.readFromFile(fin, &msg)) {
            AfxMessageBox(msg);
            delete [] msg;
        }
        fin.close();

	// load for opf tracker
	UpdateData(TRUE);  // get nsamples
	int n = m_opf_nsamples;

	if (m_method == OptimizedPF)
	{
		ifstream fin_opf(path);
		char *msg_opf;
		if (!opf.readFromFile(fin_opf, &msg_opf, n)) {
			AfxMessageBox(msg_opf);
			delete [] msg_opf;
		}
		fin_opf.close();
		// end of opf tracker

		opf.m_global.opti = m_opti;
		opf.m_global.mc_step = m_mcstep;
		opf.m_global.NeffFlag = m_neff;
	}
	else if (m_method == RegularizedPF)
	{
		// Regularized PF
		ifstream fin_rpf(path);
		char *msg_rpf;
		if (!rpf.readFromFile(fin_rpf, &msg_rpf, n)) {
			AfxMessageBox(msg_rpf);
			delete [] msg_rpf;
		}
		fin_rpf.close();

		rpf.m_global.opti = m_opti;
		rpf.m_global.mc_step = m_mcstep;
	}
	else if (m_method == UnscentedPF)
	{
		// Unscented PF
		ifstream fin_upf(path);
		char *msg_upf;
		if (!upf.readFromFile(fin_upf, &msg_upf, n)) {
			AfxMessageBox(msg_upf);
			delete [] msg_upf;
		}
		fin_upf.close();

		upf.m_global.opti = m_opti;
		upf.m_global.mc_step = m_mcstep;
	}

	
	}	
}

void CBaseTrackerDlg::OnOK() 
{
	
	CDialog::OnOK();
}



void CBaseTrackerDlg::OnOptimizedPF() 
{
//	UpdateData(TRUE); // UPDATE input data

	char *msg;
    bool ready = opf.readyToTrack(&msg);

    if (!ready) {
        AfxMessageBox(msg);
        delete [] msg;
        return;
    }

	CFileDialog saveDlg(false, "avi", NULL, 
        OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
        "AVI File (*.avi)|*.avi|All Files (*.*)|*.*||",
        this);

    // Tell the dialog to look in the working directory
    char buffer[512];
    _getcwd(buffer, 512);
    saveDlg.m_ofn.lpstrInitialDir = buffer;

    if (IDOK == saveDlg.DoModal()) {
        opf.trackAll(saveDlg.GetPathName());
    }

}

void CBaseTrackerDlg::OnSavetracker() 
{
	CFileDialog saveDlg(false, "txt", NULL, 
        OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
        "Text File (*.txt)|*.txt|All Files (*.*)|*.*||",
        this);

    // Tell the dialog to look in the working directory
    char buffer[512];
    _getcwd(buffer, 512);
    saveDlg.m_ofn.lpstrInitialDir = buffer;

    if (IDOK == saveDlg.DoModal()) {
        ofstream fout(saveDlg.GetPathName());
        opf.saveHistory(fout);
        fout.close();
    }
	
}

void CBaseTrackerDlg::OnOpfBatch() 
{
	opf.batchrun();	
}

void CBaseTrackerDlg::OnMethodTrack() 
{
	if( m_method == OptimizedPF)
	{
		
			char *msg;
			bool ready = opf.readyToTrack(&msg);
			if (!ready) {
				AfxMessageBox(msg);
				delete [] msg;
				return;
			}

			CFileDialog saveDlg(false, "avi", NULL, 
				OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
				"AVI File (*.avi)|*.avi|All Files (*.*)|*.*||",
				this);

			// Tell the dialog to look in the working directory
			char buffer[512];
			_getcwd(buffer, 512);
			saveDlg.m_ofn.lpstrInitialDir = buffer;

			if (IDOK == saveDlg.DoModal()) {
				opf.trackAll(saveDlg.GetPathName());
			}

	}

	else if (m_method == RegularizedPF)
	{
		char *msg;
		bool ready = rpf.readyToTrack(&msg);
		if (!ready) {
		AfxMessageBox(msg);
			delete [] msg;
			return;
		}

		CFileDialog saveDlg(false, "avi", NULL, 
			OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
			"AVI File (*.avi)|*.avi|All Files (*.*)|*.*||",
			this);

		// Tell the dialog to look in the working directory
		char buffer[512];
		_getcwd(buffer, 512);
		saveDlg.m_ofn.lpstrInitialDir = buffer;

		if (IDOK == saveDlg.DoModal()) {
			rpf.trackAll(saveDlg.GetPathName());
		}
	}
	else if (m_method == UnscentedPF)
	{
		char *msg;
		bool ready = upf.readyToTrack(&msg);
		if (!ready) {
		AfxMessageBox(msg);
			delete [] msg;
			return;
		}

		CFileDialog saveDlg(false, "avi", NULL, 
			OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
			"AVI File (*.avi)|*.avi|All Files (*.*)|*.*||",
			this);

		// Tell the dialog to look in the working directory
		char buffer[512];
		_getcwd(buffer, 512);
		saveDlg.m_ofn.lpstrInitialDir = buffer;

		if (IDOK == saveDlg.DoModal()) {
			upf.trackAll(saveDlg.GetPathName());
		}
	}
	
}

void CBaseTrackerDlg::OnMethodBatch() 
{
	if (m_method == OptimizedPF)
	{
		char *msg;
		bool ready = opf.readyToTrack(&msg);
		if (!ready) {
			AfxMessageBox(msg);
			delete [] msg;
			return;
		}
		opf.batchrun();
	}
	else if (m_method == RegularizedPF)
	{
		char *msg;
		bool ready = rpf.readyToTrack(&msg);
		if (!ready) {
			AfxMessageBox(msg);
			delete [] msg;
			return;
		}
		rpf.batchrun();
	}
	else if (m_method == UnscentedPF)
	{
		char *msg;
		bool ready = upf.readyToTrack(&msg);
		if (!ready) {
		AfxMessageBox(msg);
			delete [] msg;
			return;
		}
		upf.batchrun();
	}
}
