// Data types of PF
#ifndef RPFDataType_INCLUDED
#define RPFDataType_INCLUDED


#include "cv.h"

#include "Dynamics.h"
#include "EdgeAppearanceModel.h"
#include "PatchAppearanceModel.h"
#include "KinematicTree.h"
#include "Registration.h"
#include "TrackerException.h"
#include "Vector.h"

typedef struct RPFGlobalData {

	int mc_step;

	int opti;
	
	int nsamples;
	
	/** optimal kernel bandwidth */
	double h;
	
	/** Number of initial states used */
	int nInitStates;  

	/** Prior Covariance Matrix */
//	Matrix priorCov;

	/** Prior Deviation (assume the state componets are i.i.d.)*/
	Vector priorSigma;

	/** 
        Array of bools, one per sample per initial state; an element is true
        if the corresponding initial state has been set.
     */
	bool **initted;

	/** for each sample: The list of initial states; an array of length nInitStates */
    Vector **initStates;

  	/** The appearance model */
	PatchAppearanceModel eam;
	
    /** The instance of the Registration class being used */
    Registration reg;

	/** A kinametic tree to represent the tracker (not the samples) */
	KinematicTree *kt;  // called by OPF::drawResult()

} RPFGlobalData;

typedef struct RPFIterationData {

	/** The kinematic model being used */
	vector<KinematicTree*> kt_new, kt_old;
	
	/** The dynamic model */
    vector<Dynamics*> dyn_new, dyn_old;
	
	double *sample_weights, *cumul_prob_array, largest_cumulative_prob;

	/** Current value of the objective function */
	double *curScore;  // directly related to sample_weights
} RPFIterationData;

#endif  // ! def PFDataType_INCLUDED