#include "stdafx.h"
#include "BaseTracker.h"
#include "TrackingDlg.h"

#include <direct.h> // for getcwd()

#include "Project.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// TrackingDlg dialog


TrackingDlg::TrackingDlg(Project *pProject, CWnd* pParent /*=NULL*/)
	: CDialog(TrackingDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(TrackingDlg)
	m_background = -1;
	m_foreground = -1;
	//}}AFX_DATA_INIT

    project = pProject;
}


void TrackingDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(TrackingDlg)
	DDX_Radio(pDX, IDC_COLOR_BACKGROUND, m_background);
	DDX_Radio(pDX, IDC_EDGES_FOREGROUND, m_foreground);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(TrackingDlg, CDialog)
	//{{AFX_MSG_MAP(TrackingDlg)
	ON_BN_CLICKED(IDC_STEP_FORWARD, OnStepForward)
	ON_BN_CLICKED(IDC_STEP_BACK, OnStepBack)
	ON_BN_CLICKED(IDC_FRAME_FORWARD, OnFrameForward)
	ON_BN_CLICKED(IDC_FRAME_BACK, OnFrameBack)
	ON_BN_CLICKED(IDC_EDGES_FOREGROUND, OnForeground)
	ON_BN_CLICKED(IDC_COLOR_BACKGROUND, OnBackground)
	ON_BN_CLICKED(IDC_SAVE_HISTORY, OnSaveHistory)
	ON_BN_CLICKED(IDC_EDGES_BACKGROUND, OnBackground)
	ON_BN_CLICKED(IDC_DIST_BACKGROUND, OnBackground)
	ON_BN_CLICKED(IDC_BOXES_FOREGROUND, OnForeground)
	ON_BN_CLICKED(IDC_STICK_FOREGROUND, OnForeground)
	ON_BN_CLICKED(IDC_TRACK_ALL, OnTrackAll)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// TrackingDlg message handlers


void TrackingDlg::OnCancel() 
{
    project->hide();	
	CDialog::OnCancel();
}

void TrackingDlg::OnStepForward() 
{
    if(project->nextStep()) {
        AfxMessageBox("Done");
    }
}

void TrackingDlg::OnStepBack() 
{
    if (project->prevStep()) {
        AfxMessageBox("Already on first step.");
    }
}

void TrackingDlg::OnFrameForward() 
{
    if(project->nextFrame()) {
        AfxMessageBox("Done");
    }
}

void TrackingDlg::OnFrameBack() 
{
    if (project->prevFrame()) {
        AfxMessageBox("Already on first step.");
    }
}

void TrackingDlg::OnBackground() 
{
    UpdateData(TRUE);
    
    if (this->m_background == IDC_COLOR_BACKGROUND - IDC_COLOR_BACKGROUND) {
        project->background = project->COLOR;
    }
    else if (m_background == IDC_EDGES_BACKGROUND - IDC_COLOR_BACKGROUND) {
        project->background = project->EDGES;
    }
    else {
        project->background = project->DISTANCE;
    }

    project->display();
}

void TrackingDlg::OnForeground() 
{
    UpdateData(TRUE);
    
    if (this->m_foreground == IDC_EDGES_FOREGROUND - IDC_EDGES_FOREGROUND) {
        project->foreground = project->PIXELS;
    }
    else if (m_foreground == IDC_BOXES_FOREGROUND - IDC_EDGES_FOREGROUND) {
        project->foreground = project->BOXES;
    }
    else {
        project->foreground = project->LINES;
    }

    project->display();
}

BOOL TrackingDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
    CheckRadioButton(IDC_COLOR_BACKGROUND, IDC_DIST_BACKGROUND, 
                     IDC_COLOR_BACKGROUND);
    CheckRadioButton(IDC_EDGES_FOREGROUND, IDC_STICK_FOREGROUND,
                     IDC_EDGES_FOREGROUND);
    
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void TrackingDlg::OnSaveHistory() 
{
    CFileDialog saveDlg(false, "txt", NULL, 
        OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
        "Text File (*.txt)|*.txt|All Files (*.*)|*.*||",
        this);

    // Tell the dialog to look in the working directory
    char buffer[512];
    _getcwd(buffer, 512);
    saveDlg.m_ofn.lpstrInitialDir = buffer;

    if (IDOK == saveDlg.DoModal()) {
        ofstream fout(saveDlg.GetPathName());
        project->saveHistory(fout);
        fout.close();
    }
}

void TrackingDlg::OnTrackAll() 
{
    CFileDialog saveDlg(false, "avi", NULL, 
        OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
        "AVI File (*.avi)|*.avi|All Files (*.*)|*.*||",
        this);

    // Tell the dialog to look in the working directory
    char buffer[512];
    _getcwd(buffer, 512);
    saveDlg.m_ofn.lpstrInitialDir = buffer;

    if (IDOK == saveDlg.DoModal()) {
        project->trackAll(saveDlg.GetPathName());
    }
}
