/******************************************
Implementation of Unscented Particle Filter
*******************************************/

#include "UPF.h"

#include <cstring>
#include <vector>
using namespace std;

#include "cv.h"
#include "highgui.h"

#include "AVIWriter.h"
#include "ImgFun.h"
#include "IOFun.h"
#include "MathFun.h"
#include "Dynamics.h"
#include "ARMADynamics.h"
#include "ConstVelDynamics.h"
#include "NoDynamics.h"
#include "SSIDDynamics.h"


#include "utility.h"
#include "time.h"

// #include "tnt.h"
//#include "tnt_array2d.h" 
//#include "jama_cholesky.h" 
//#include "jama_svd.h" 

//using namespace JAMA;

//////////////////
//  Constructor
//////////////////
UPF::UPF()
{
//	m_global.initStates = NULL;
//	m_global.initted = NULL;
	m_global.nInitStates = 0;
	m_global.kt = NULL;
//	m_global.priorSigma = 2;
	m_global.mc_step = 0;
	m_global.opti = 0;
	
/*	m_iteration.dyn_new = NULL;
	m_iteration.dyn_old = NULL;
	m_iteration.kt_new = NULL;
	m_iteration.kt_old = NULL;
*/	m_iteration.cumul_prob_array = NULL;
	m_iteration.curScore = NULL;
	m_iteration.sample_weights = NULL;
	m_iteration.tmp_weights = NULL;

	START_COLOR = CV_RGB(0, 255, 0);
    PRED_COLOR = CV_RGB(255, 255, 255);
    STEP_COLOR = CV_RGB(255, 255, 0);
    FINAL_COLOR = CV_RGB(255, 0, 0);

    eamInitted = false;

    displayImg = NULL;
	frame = NULL;
    aviFilename = ktFilename = dynFilename = NULL;
    background = COLOR;
    foreground = PIXELS;
}

//////////////////
// Destructor
//////////////////
UPF::~UPF()
{
	int i;
	if (m_iteration.dyn_new.size() != 0) 
	{
	for (i = 0; i < m_iteration.dyn_new.size(); i++) {
		if (m_iteration.dyn_new[i] != NULL)
			delete m_iteration.dyn_new[i];
	}
	}
	if (m_iteration.dyn_old.size() != 0) 
	{
	for (i = 0; i < m_iteration.dyn_old.size(); i++) {
		if (m_iteration.dyn_old[i] != NULL)
			delete m_iteration.dyn_old[i];
	}
	}
	if (m_iteration.kt_new.size() != 0) 
	{
		for (i = 0; i<m_iteration.kt_new.size(); i++)
			delete m_iteration.kt_new[i];
	}
	if (m_iteration.kt_old.size() != 0) 
	{
		for (i = 0; i<m_iteration.kt_old.size(); i++)
			delete m_iteration.kt_old[i];
	}
	if (m_iteration.cumul_prob_array != NULL) {
		delete m_iteration.cumul_prob_array;
		m_iteration.cumul_prob_array = NULL;
	}
	if (m_iteration.curScore != NULL) {
		delete m_iteration.curScore;
		m_iteration.curScore = NULL;
	}
	if (m_iteration.sample_weights != NULL) {
		delete m_iteration.sample_weights;
		m_iteration.sample_weights = NULL;
	}
	if (m_iteration.tmp_weights != NULL) {
		delete m_iteration.tmp_weights;
		m_iteration.tmp_weights = NULL;
	}

	if (aviFilename != NULL) {
        delete [] aviFilename;
		aviFilename = NULL;
    }
    if (ktFilename != NULL) {
        delete [] ktFilename;
		ktFilename = NULL;
    }
	if (dynFilename != NULL) {
        delete [] dynFilename;
		dynFilename = NULL;
    }

	if (m_global.initStates.size() != 0)
	{
		for (i = 0; i < m_global.initStates.size(); i++)
			delete [] m_global.initStates[i];
	}
	if (m_global.initted.size() != 0)
	{
		for (i = 0; i < m_global.initted.size(); i++)
			delete [] m_global.initted[i];
	}
	if (m_global.kt != NULL) {
		delete m_global.kt;
		m_global.kt = NULL;
	}

	if (displayImg != NULL) cvReleaseImage(&displayImg);
	if (frame != NULL) cvReleaseImage(&frame);
}
 
/* From here on is generic Condensation regardless of the form of
   model or observation used. All of the model-specific routines are
   found in model_specific.c */
int UPF::pick_base_sample()
{
	double choice = uniform_random() * m_iteration.largest_cumulative_prob;
	int low, middle, high;

	low = 0;
	high = m_global.nsamples;

	while (high>(low+1)) {
		middle = (high+low)/2;
		if (choice > m_iteration.cumul_prob_array[middle])
			low = middle;
		else high = middle;
	}

	return low;
}

void UPF::predict_sample_position(int new_sample, int old_sample)
{
	// Make a prediction via the LDS model.  This will serve as starting
    // point for the image registration.

	prevState = m_iteration.kt_old[old_sample]->getState();
	// copy dyn_old, 'cause it is sampling with replacement
	m_iteration.dyn_new[new_sample]->copyStates(m_iteration.dyn_old[old_sample]);
	m_iteration.dyn_new[new_sample]->predict();
	pred = toFullState(m_iteration.dyn_new[new_sample]->observe(), prevState);
//		+ m_global.priorSigma.vectorproduct(randn(prevState.len())) * 2;
	m_iteration.kt_new[new_sample]->setState(pred);

}

void UPF::sample_move(int new_sample, int old_sample)
{
	int n, i, j;

	// assume the transition kernel to be a Gaussian
	// Covariance = 2 * prior Covariance
//	Vector processSigma = m_global.priorSigma * 2;
	
	// proposal state
	prevState = m_iteration.kt_old[old_sample]->getState();
//	Vector xProp = prevState + processSigma.vectorproduct(randn(prevState.len()));
	m_iteration.p_old[old_sample].SVD(U, V, D);
	V.setIdentity();
	for ( i = 0; i < D.cols(); i++)
		V.set(i, i, sqrt(D.get(i,i)));
	Vector xProp = m_iteration.dyn_old[old_sample]->observe() + V * randn(D.cols());

	// evaluate
	double curScoreProp, curScorePred;
	double ratio, priorPred, priorProp, Pred, Prop;
	double u = uniform_random();
/*	m_global.kt->setState(xProp);
	frame = avi.getGrayFrame(frameIdx);
	m_global.reg.prepForRegistration(frame, m_global.eam, *m_global.kt, curScoreProp);
	m_global.reg.prepForRegistration(frame, m_global.eam, *(m_iteration.kt_new[new_sample]), curScorePred);
	if (u <= __min(1, exp((curScorePred-curScoreProp)/10))) {
		// accept the move
		m_iteration.kt_new[new_sample]->setState(xProp);
		n = naccept.size();
		naccept[n-1] ++;
	}
	else {
		// accept the prediction, don't move
		// do nothing
		n = nreject.size();
		nreject[n-1] ++;
	}
	cvReleaseImage(&frame);
*/

	// scaled Unscented Transform
	sigmaPts.clear();
	sigmaPts.push_back(xProp);

	n = sigmaPts[0].len();
	P = m_iteration.p_old[old_sample] * (n + KAPPA);
	P.SVD(U, V, D);
	P.setIdentity();
	for (i = 0; i < n; i++)
		P.set(i, i, sqrt(D.get(i, i)));  // now P = sqrt(D)
	V = U * P;
	for (i = 0; i < n; i++)
	{
		for (j = 0; j < n; j++)
			tmp.set(j, V.get(j, i));  // ith col of V
		
		sigmaPts.push_back(sigmaPts[0] + tmp);
		sigmaPts.push_back(sigmaPts[0] - tmp);
	}

	// propagate
	double score, obs;

	// initialize mx_move, my, PxPred, K, Py, Pxy
	mx_move.setZero();
	my = 0;
	Py = 0;
	Pxy.setZero();
	PxPred.setZero();

	sigmaObs.clear();

	// copy vel, x
	m_iteration.dyn_new[new_sample]->copyStates(m_iteration.dyn_old[old_sample]);

	pred = m_iteration.kt_old[old_sample]->getState();

	for (i = 0 ; i < sigmaPts.size(); i++) {
		m_iteration.dyn_new[new_sample]->setState(sigmaPts[i]);  // update x
		sigmaPts[i] = m_iteration.dyn_new[new_sample]->predict();
		mx_move += sigmaPts[i];
		
		m_iteration.kt_new[new_sample]->setState(toFullState(sigmaPts[i], pred));
		
		m_global.reg.prepForRegistration(displayImg, m_global.eam, *(m_iteration.kt_new[new_sample]), score);
		sigmaObs.push_back(exp(- score/ step));
		my += sigmaObs[i];
	}
	mx_move = mx_move * sigmaWts[1] + sigmaPts[0] * (sigmaWts[0] - sigmaWts[1]);
	my = my * sigmaWts[1] + sigmaObs[0] * (sigmaWts[0] - sigmaWts[1]);

	// "true" observation, corresponding to mx_move
	m_iteration.kt_new[new_sample]->setState(toFullState(mx_move, pred));
	m_global.reg.prepForRegistration(displayImg, m_global.eam, *(m_iteration.kt_new[new_sample]), score);
	obs = exp(-score/ step);  // observation

	for (i = 0 ; i < sigmaPts.size(); i++) {
		sigmaPts[i] = sigmaPts[i] - mx_move;
		sigmaObs[i] = sigmaObs[i] - my;
	}

	// PxPred, Py, Pxy
	for (i = 0 ; i < sigmaPts.size(); i++) {
		// calcualte PxPred
		vector2matrix(sigmaPts[i], mdiff);
		PxPred = mdiff * mdiff.transpose() + PxPred;
		Py += sigmaObs[i] * sigmaObs[i];
		Pxy = sigmaPts[i] * sigmaObs[i]  + Pxy;
	}
	vector2matrix(sigmaPts[0], mdiff);
	PxPred = PxPred * sigmaWts[1] + mdiff * (mdiff.transpose() * (sigmaWts[sigmaWts.size()-1] - sigmaWts[1]));
	Py = Py * sigmaWts[1] + sigmaObs[0]*sigmaObs[0] * (sigmaWts[sigmaWts.size()-1] - sigmaWts[1]);
	Pxy = Pxy * sigmaWts[1] + sigmaPts[0] * (sigmaObs[0] * (sigmaWts[sigmaWts.size()-1] - sigmaWts[1]));

	// update
	if (Py == 0)
		Py = Py + 0.0001;
	K = Pxy *(1 / Py);
	mx_move = mx_move + K * (obs - my);
	vector2matrix(K, mdiff);
	P = PxPred - (mdiff * Py) * mdiff.transpose();
	if (P.det() == 0) {
		U.setIdentity();
		P = P + U * 0.0001;
	}
	// end of propagation

	// generate sample xProp from N(mx_move, P)
	P.SVD(U, V, D);
	V.setIdentity();
	for (i = 0; i < n; i++)
		V.set(i, i, sqrt(D.get(i, i)));
	sigmaX = U * V;

	tmp = mx_move + sigmaX * randn(mx_move.len());
	xProp = tmp;

	m_global.kt->setState(toFullState(xProp, prevState));

	// evaluate weight
	if (m_global.opti == 1)    // optimization case
	{
		m_global.reg.prepForRegistration(displayImg, m_global.eam, *m_global.kt, curScoreProp);
		while (!m_global.reg.oneIteration(m_global.eam, *m_global.kt, curScoreProp)) {
		}
		curScorePred = m_iteration.curScore[new_sample];
		if (u <= __min(1, exp((curScorePred-curScoreProp)/10))) {
			// accept the move
			xProp = m_global.kt->getState();
			m_iteration.kt_new[new_sample]->setState(xProp);
			m_iteration.curScore[new_sample] = curScoreProp;
			m_iteration.p_new[new_sample] = P;
			mx = mx_move;
			n = naccept.size();
			naccept[n-1]++;
		}
		else {
			// accept the prediction, don't move
			// do nothing
			n = nreject.size();
			nreject[n-1] ++;
		}
	}
	else                       // non-optimization case
	{
		m_global.reg.prepForRegistration(displayImg, m_global.eam, *m_global.kt, curScoreProp);
		curScorePred = m_iteration.curScore[new_sample];

		invP = m_iteration.p_old[old_sample].inv();
		tmp = m_iteration.dyn_new[new_sample]->observe() - m_iteration.dyn_old[old_sample]->observe();
		priorPred = (invP * tmp) * tmp;
		tmp = xProp - m_iteration.dyn_old[old_sample]->observe();
		priorProp = (invP * tmp) * tmp;

		ratio = exp(0.5*(priorPred - priorProp));

		tmp = m_iteration.dyn_new[new_sample]->observe() - mx;
		Pred = exp(-0.5 * ((m_iteration.p_new[new_sample].inv() * tmp) * tmp )) / 
			sqrt(m_iteration.p_new[new_sample].det());
		tmp = xProp - mx_move;
		Prop = exp(-0.5 * ((P.inv() * tmp) * tmp )) / 
			sqrt(P.det());

		ratio = ratio * (Pred / Prop) * exp((curScorePred-curScoreProp)/step);

		if (u <= __min(1, ratio)) {
			// accept the move
			xProp = m_global.kt->getState();
			m_iteration.kt_new[new_sample]->setState(xProp);
			m_iteration.curScore[new_sample] = curScoreProp;
			m_iteration.p_new[new_sample] = P;
			mx = mx_move;
			n = naccept.size();
			naccept[n-1] ++;
		}
		else {
			// accept the prediction, don't move
			// do nothing
			n = nreject.size();
			nreject[n-1] ++;
		}
	}

}


/** sample move is considered to be accepted or not based on registration result. */
void UPF::sample_move_registered(int new_sample, int old_sample)
{
	int n;
	// assume the transition kernel to be a Gaussian
	// Covariance = 2 * prior Covariance
	Vector processSigma = m_global.priorSigma * 2;
	
	// proposal state
	prevState = m_iteration.kt_old[old_sample]->getState();
	Vector xProp = prevState + processSigma.vectorproduct(randn(prevState.len()));

	// evaluate
	double curScoreProp, curScorePred;
	double u = uniform_random();
	m_global.kt->setState(xProp);
	frame = avi.getGrayFrame(frameIdx);
	if (m_global.opti == 1) {
		m_global.reg.prepForRegistration(frame, m_global.eam, *m_global.kt, curScoreProp);
		while (!m_global.reg.oneIteration(m_global.eam, *m_global.kt, curScoreProp)) {
		}
	}
	else {
		curScoreProp = m_global.eam.getScore(m_global.kt);
	}
	curScorePred = m_iteration.curScore[new_sample]; // score of predicted sample being registered
	if (u <= __min(1, exp((curScorePred-curScoreProp)/10))) {
		// accept the move
		xProp = m_global.kt->getState();
		m_iteration.kt_new[new_sample]->setState(xProp);
		m_iteration.curScore[new_sample] = curScoreProp;
		n = naccept.size();
		naccept[n-1]++;
	}
	else {
		// accept the prediction, don't move
		// do nothing
		n = nreject.size();
		nreject[n-1] ++;
	}
	cvReleaseImage(&frame);
}

/**
	perform image registration
	update kt, dyn
	*/

void UPF::registration(int new_sample)
{
	frame = avi.getGrayFrame(frameIdx);
	if (m_global.opti == 1) {
	m_global.reg.prepForRegistration(frame, m_global.eam, *(m_iteration.kt_new[new_sample]), m_iteration.curScore[new_sample]);
	while (!m_global.reg.oneIteration(m_global.eam, *(m_iteration.kt_new[new_sample]), m_iteration.curScore[new_sample])) {
//		m_iteration.kt_new[new_sample]->drawLines(image, CV_RGB(255, 255, 0));
//		outputavi.append(image);
	}
	}
	else
		m_global.reg.prepForRegistration(frame, m_global.eam, *(m_iteration.kt_new[new_sample]), m_iteration.curScore[new_sample]);
	cvReleaseImage(&frame);
}

/**
	Display the [new_sample]th prediction
	*/
void UPF::drawPred(int new_sample)
{
	frame = avi.getFrame(frameIdx);
	m_iteration.kt_new[new_sample]->drawLines(frame, CV_RGB(255, 0, 0));
	sprintf(text, "prediction: sample %d", new_sample);
    cvPutText(frame, text, cvPoint(20, 20), &font, CV_RGB(255, 0, 0));
	outputAVI.append(frame);
	cvShowImage(UPF_WINDOW_NAME, frame);
	cvReleaseImage(&frame);
}

/** 
	draw final registration result
	*/
void UPF::drawRegistr(int new_sample)
{
	frame = avi.getFrame(frameIdx);
	m_iteration.kt_new[new_sample]->drawLines(frame, CV_RGB(0, 255, 0));
	sprintf(text, "registration result: sample %d", new_sample);
    cvPutText(frame, text, cvPoint(20, 20), &font, CV_RGB(0, 255, 0));
    outputAVI.append(frame);
	cvShowImage(UPF_WINDOW_NAME, frame);
	cvReleaseImage(&frame);
}

void UPF::drawAllSamples()
{
	frame = avi.getFrame(frameIdx);
	for (int i = 0; i < m_global.nsamples; i++)
	{
		m_iteration.kt_new[i]->drawLines(frame, CV_RGB(0, 255, 0));
	}
	m_global.kt->drawLines(frame, CV_RGB(255, 0, 0));
	sprintf(text, "draw all samples");
	cvPutText(frame, text, cvPoint(20, 20), &font, CV_RGB(0, 255, 0));
	outputAVI.append(frame);
	cvShowImage(UPF_WINDOW_NAME, frame);
	cvReleaseImage(&frame);
}

void UPF::drawResult()
{
	frame = avi.getFrame(frameIdx);
	m_global.kt->setState(tracker);
	m_global.kt->drawLines(frame, CV_RGB(0, 255, 0));
	sprintf(text, "draw the tracker");
	cvPutText(frame, text, cvPoint(20, 20), &font, CV_RGB(0, 255, 0));
	outputAVI.append(frame);
	cvShowImage(UPF_WINDOW_NAME, frame);
	cvReleaseImage(&frame);
}

void UPF::drawCorrection(int new_sample)
{
	frame = avi.getFrame(frameIdx);
	sprintf(text, "correction for sample %d", new_sample);
	corrected = toFullState(m_iteration.dyn_new[new_sample]->observe(), m_iteration.kt_new[new_sample]->getState());
	m_iteration.kt_new[new_sample]->setState(corrected);
    frame = avi.getFrame(frameIdx);
    m_iteration.kt_new[new_sample]->drawLines(frame, CV_RGB(0, 0, 255));
    cvPutText(frame, text, cvPoint(20, 20), &font, CV_RGB(255, 0, 0));
    outputAVI.append(frame);
	cvShowImage(UPF_WINDOW_NAME, frame);
	cvReleaseImage(&frame);
}

void UPF::calculate_base_weights()
{
	int i;
	double cumul_total;

	if ((m_global.NeffFlag == 1) && (Neff >= Nthres)) {
		cumul_total = 0;
		for (i = 0; i < m_global.nsamples; i++)
		{	
			m_iteration.sample_weights[i] *= m_iteration.tmp_weights[i];
			cumul_total += m_iteration.sample_weights[i];
		}
		m_iteration.largest_cumulative_prob = cumul_total;
	}
	else {
		cumul_total = 0;
		for (i = 0; i < m_global.nsamples; i++)
		{	
			m_iteration.sample_weights[i] = m_iteration.tmp_weights[i];
			cumul_total += m_iteration.sample_weights[i];
		}
		m_iteration.largest_cumulative_prob = cumul_total;
	}

    // normalize the weights
	cumul_total = 0.0;
    for ( i = 0; i < m_global.nsamples; ++i)
	{
	    m_iteration.sample_weights[i] = m_iteration.sample_weights[i] / m_iteration.largest_cumulative_prob;

		m_iteration.cumul_prob_array[i] = cumul_total;
        cumul_total += m_iteration.sample_weights[i];
	}
    m_iteration.largest_cumulative_prob = cumul_total;

}

void UPF::processFrame()
{
	int n, base;

	// swap data structure of old samples and new samples
	m_iteration.dyn_new.swap(m_iteration.dyn_old);
	m_iteration.kt_new.swap(m_iteration.kt_old);
	m_iteration.p_new.swap(m_iteration.p_old);

	// Push previous state through process model
	
	// initialize naccept and nreject at frameIdx
	naccept.push_back(0);
	nreject.push_back(0);

	if (m_global.NeffFlag == 1) {
		// calculate Neff
		Neff = 0;
		for (n = 0; n < m_global.nsamples; ++n)
			Neff += pow(m_iteration.sample_weights[n],2);
		Neff = 1 / Neff;
	}

	displayImg = avi.getGrayFrame(frameIdx);
	m_global.eam.setImage(displayImg);

	if (m_global.mc_step == 0)  // no mc_step
		for (n = 0; n < m_global.nsamples; ++n)
		{
			// The new sample position is computed by sampling from the
			// prediction density p(x_t|x_t-1 = base).
			if ((m_global.NeffFlag == 1) && (Neff >= Nthres))
				base = n;
			else
				base = pick_base_sample();
			
			// prediction - process model specific routine
			UKF(n, base);

			// draw prediction
//			drawPred(n);

			// registration
			registration(n);

			// draw registration result
//			drawRegistr(n);
			
			// correct dynamic model
			m_iteration.dyn_new[n]->correct(fromFullState(m_iteration.kt_new[n]->getState()));

			// draw correction
//			drawCorrection(n);

			evaluate_weights(n, base);

		}
	else   // mc move
		for (n = 0; n < m_global.nsamples; ++n)
		{
			// The new sample position is computed by sampling from the
			// prediction density p(x_t|x_t-1 = base).
			base = pick_base_sample();
			
			// prediction - process model specific routine
			UKF(n, base);

			// draw prediction
	//		drawPred(n);

			// registration
			registration(n);

			// optional: optimized sample move
			sample_move_registered(n, base);
//			sample_move(n, base);

			// draw registration result
	//		drawRegistr(n);

			// correct dynamic model
			m_iteration.dyn_new[n]->correct(fromFullState(m_iteration.kt_new[n]->getState()));

			// draw correction
	//		drawCorrection(n);

			evaluate_weights(n, base);

		}

	cvReleaseImage(&displayImg);

	// Apply Bayesian measurement weighting
	// This part computes the weights by evaluating
	// the observation density at each of the positions.
	calculate_base_weights();


//	calculate_shape_parameters();

	calculate_shape_parameters_MAP();
	
}

void UPF::calculate_shape_parameters_MAP()
{
	double temp = 0;

	for (int i=0; i<m_global.nsamples; i++) {
		if (temp < m_iteration.sample_weights[i]) {
			temp = m_iteration.sample_weights[i];
			tracker = m_iteration.kt_new[i]->getState();  // full state
		} else {
		} // end of if-else
	} // end of for*/

	m_global.kt->setState(tracker);
	trackerScore = m_global.eam.getScore(m_global.kt);
	jointCtrs.clear();
	m_global.kt->getJointCenters(jointCtrs);
}

void UPF::calculate_shape_parameters()
{
	int len = m_iteration.kt_new[0]->getStateLen();
//	tracker.initialize(len);
	tracker.setZero();

	for (int i = 0; i < m_global.nsamples; i++)
		tracker += m_iteration.kt_new[i]->getState() * m_iteration.sample_weights[i];

	m_global.kt->setState(tracker);
	jointCtrs.clear();
	m_global.kt->getJointCenters(jointCtrs);
}


void UPF::set_up_prior_conditions()
{
	int i, j;
	Matrix I;
	i = m_iteration.dyn_new[0]->observe().len();
	I.initialize(i, i);

	// read prior deviation of states from file
	ifstream fin_scov("..\\sample projects\\initCov_GreenLeg.txt"); // also used for dance
	if (fin_scov.is_open())
		for (i = 0; i < I.rows(); i++)
			for (j = 0; j < I.cols(); j++)
				I.set(i, j, IOFun::readDoubleWithComments(fin_scov));
		else
			cout << "could not load prior deviation for init states!" << endl;
	fin_scov.close();
	if (I.det() == 0) {
		P.setIdentity();
		I = I + P * 0.0001;
	}

	m_iteration.p_new.clear();
	m_iteration.p_old.clear();
	for (i = 0; i < m_global.nsamples; i++)
	{
		m_iteration.curScore[i] = 0;
		m_iteration.cumul_prob_array[i] = (double) i / m_global.nsamples;
		m_iteration.sample_weights[i] = 1.0 / m_global.nsamples;

		// initialize covariance matrix p_old, p_new
		m_iteration.p_new.push_back(I);
		m_iteration.p_old.push_back(I);
	}
	m_iteration.largest_cumulative_prob = 1;

}

void UPF::initialize_sample_weights()
{
	/** allocate memory for m_iteration.sample_weights, 
	m_iteration.cumul_prob_array, m_iteration.curScore.  */
	if (m_iteration.sample_weights != NULL) delete m_iteration.sample_weights;
	if (m_iteration.cumul_prob_array != NULL) delete m_iteration.cumul_prob_array;
	if (m_iteration.curScore != NULL) delete m_iteration.curScore;
	if (m_iteration.tmp_weights != NULL) delete m_iteration.tmp_weights;
	m_iteration.sample_weights = new double[m_global.nsamples];
	m_iteration.cumul_prob_array = new double[m_global.nsamples];
	m_iteration.curScore = new double[m_global.nsamples];
	m_iteration.tmp_weights = new double[m_global.nsamples];

	set_up_prior_conditions();
}

void UPF::vector2matrix(Vector &v, Matrix &m)
{
	assert(v.len() == m.rows() && m.cols() == 1);
	for (int i = 0; i < v.len(); i++)
		m.set(i, 0, v.get(i));
}


/*********************************************************/

/**
    Opens a new AVI file.  
    @param fname filename of AVI file to open
    @param msg if there is an error, an error message is returned here
    @return true if opening is successful, otherwise return 
        false and an error message in msg.
 */
bool UPF::openAVI(const char *fname, char **msg) {
    if (aviFilename != NULL) {
        delete [] aviFilename;
    }
    aviFilename = new char[strlen(fname) + 1];
    strcpy(aviFilename, fname);

    avi.open(fname);

    return true;
}


/**
    Opens a dynamic model.
    @param fname filename of dynamic model file to open
    @param msg if there is an error, an error message is returned here
    @return true if successful; otherwise return false and 
        an error message in msg.
 */
bool UPF::openDynamics(const char *fname, char **msg) {
    if (dynFilename != NULL) {
        delete [] dynFilename;
    }
    dynFilename = new char[strlen(fname) + 1];
    strcpy(dynFilename, fname);

	int i, j;
    
	if (m_iteration.dyn_new.size() != 0) {
	for (j = 0; j < m_iteration.dyn_new.size(); j++) {
		if (m_iteration.dyn_new[j] != NULL)
			delete m_iteration.dyn_new[j];
    }
	}
	if (m_iteration.dyn_old.size() != 0) {
	for (j = 0; j < m_iteration.dyn_old.size(); j++) {
		if (m_iteration.dyn_old[j] != NULL)
			delete m_iteration.dyn_old[j];
    }
	}
    
    ifstream fin(fname);
//    if ((m_iteration.dyn_new == NULL) || (m_iteration.dyn_old == NULL)) {
        // Read first line, which tells us what sort of dynamic model
        // we have
        int len = 256;
        char *buf = new char[len];
        IOFun::readStringWithComments(fin, buf, len);

        if (strcmp(buf, "ARMA") == 0) {
			for (j = 0; j < m_global.nsamples; j++) {
				m_iteration.dyn_new.push_back(new ARMADynamics);
				m_iteration.dyn_old.push_back(new ARMADynamics);
			}
		}
        else if (strcmp(buf, "CONST") == 0) {
			for (j = 0; j < m_global.nsamples; j++) {
				m_iteration.dyn_new.push_back(new ConstVelDynamics);
				m_iteration.dyn_old.push_back(new ConstVelDynamics);
			}

        }
        else if (strcmp(buf, "SSID") == 0) {
			for (j = 0; j < m_global.nsamples; j++) {
				m_iteration.dyn_new.push_back(new SSIDDynamics);
				m_iteration.dyn_old.push_back(new SSIDDynamics);
			}
        }
        else if (strcmp(buf, "ZERO") == 0) {
			for (j = 0; j < m_global.nsamples; j++) {
				m_iteration.dyn_new.push_back(new NoDynamics);
				m_iteration.dyn_old.push_back(new NoDynamics);
			}
        }
        else {
			m_iteration.dyn_new.clear();
			m_iteration.dyn_old.clear();
            // unknown type
        //  m_iteration.dyn_new = NULL;
		//	m_iteration.dyn_old = NULL;
        }

        delete [] buf;
 //   }
    
    if ((m_iteration.dyn_new.size() == 0) || (m_iteration.dyn_old.size() == 0)) {
        // Unknown file type
        fin.close();
        *msg = new char[256];
        sprintf(*msg, "Unknown dynamics filetype.");
        return false;
    }

	long filestart = fin.tellg();
	for (j = 0; j < m_global.nsamples; j++)
	{
		fin.seekg(filestart);
		m_iteration.dyn_new[j]->readFromFile(fin);
		fin.seekg(filestart);
		m_iteration.dyn_old[j]->readFromFile(fin);
	}
    fin.close();


    // Setup the initial states.  Copy as many initial states as
    // have already been set.
    m_global.nInitStates = m_iteration.dyn_new[0]->getNInitStates(); // ninitstates is constant for all dyns.
//	bool **newInitted = new bool* [m_global.nsamples];
//	Vector **newInitStates = new Vector* [m_global.nsamples];


	for ( j = 0; j < m_global.nsamples; j++)
	{
		m_global.initted.push_back(new bool[m_global.nInitStates]);
		m_global.initStates.push_back(new Vector[m_global.nInitStates]);
	}

    for (j = 0; j < m_global.nsamples; j++)
	for (i = 0; i < m_global.nInitStates; i++) {
        m_global.initted[j][i] = false;
    }

    return true;
}


/**
   If user has initialized all necessary data, prepares for first step
   of tracking and returns true.  Otherwise, returns false and returns
   a message describing the problem in msg.
    @param msg if there is an error, an error message is returned here
 */
bool UPF::readyToTrack(char **msg) {

	int i, j;

    if (m_iteration.kt_new.size() == 0 || m_iteration.kt_old.size() == 0) {
        *msg = new char[256];
        sprintf(*msg, "Need to specify a kinematic model.");
        return false;
    }
    if (!avi.isOpen()) {
        *msg = new char[256];
        sprintf(*msg, "Need to open an AVI file.");
        return false;
    }
    if (!m_global.eam.initialized()) {
        *msg = new char[256];
        sprintf(*msg, "Need to initialize the appearance model.");
        return false;
    }
    if ((m_iteration.dyn_new.size() == 0) || (m_iteration.dyn_old.size() == 0)) {
        *msg = new char[256];
        sprintf(*msg, "Need to open a dynamic model.");
        return false;
    }
	for (j = 0; j < m_global.nsamples; j++)
    for (i = 0; i < m_global.nInitStates; i++) {
        if (!m_global.initted[j][i]) {
            *msg = new char[256];
            sprintf(*msg, "This dynamic model requires %d initial states.  All of them have not yet been specified.",
                    m_global.nInitStates);
            return false;
        }
    }

    
    // Everything is ok; do initialization
    
    // Initialize history
    history.erase(history.begin(), history.end());
    frameIdx = m_global.nInitStates - 1;

    // Initialize dynamic model hidden state
    vector<Vector> ys;
	for (j = 0; j < m_global.nsamples; j++) {
    for (i = 0; i < m_global.nInitStates; i++) {
        ys.push_back(fromFullState(m_global.initStates[j][i]));
    }
	m_iteration.dyn_new[j]->estimateX0(ys);
	m_iteration.dyn_old[j]->estimateX0(ys);
	
	ys.clear();
   	}


    // Initialize kinematic state
	for (j = 0; j < m_global.nsamples; j++)
	{
		m_iteration.kt_new[j]->setState(m_global.initStates[j][m_global.nInitStates-1]);
		m_iteration.kt_old[j]->setState(m_global.initStates[j][m_global.nInitStates-1]);
	}

	// initialize the kinematic tree for tracker state
//	m_global.kt->setState(m_global.initStates[0][m_global.nInitStates-1]);

	// initialize mx, PxPred, Pxy, K
	i = m_iteration.dyn_new[0]->observe().len();
	mx.initialize(i);
	Pxy.initialize(i);
	K.initialize(i);
	PxPred.initialize(i, i);
	sigmaX.initialize(i, i);
	U.initialize(i, i);
	V.initialize(i, i);
	D.initialize(i, i);
	P.initialize(i, i);
	tmp.initialize(i);
	mdiff.initialize(i, 1);
	

	if (m_global.mc_step != 0) {
		mx_move.initialize(i);
		invP.initialize(i,i);
	}

	// assign values to sigmaWts
	sigmaWts.clear();
	sigmaWts.push_back((double)KAPPA / (double)(i + KAPPA));
	sigmaWts.push_back(0.5/(i + KAPPA));
	sigmaWts.push_back(sigmaWts[0] + (1- ALPHA^2 + BETA));


	i = m_iteration.kt_new[0]->getStateLen();
	tracker.initialize(i);
	tracker = *(m_global.initStates[0]);

	if (m_global.NeffFlag == 1)
		Nthres = (double) m_global.nsamples / 2.0;

    // Make sure output window is closed
    hide();
    
    return true;
}

/**
   Tracks all frames of the video and saves the output as a 
   video in the file specified by filename.  Does not display
   any output along the way.
   @param fname name of AVI file to write output to.
 */
void UPF::trackAll(const char *fname) {
 
    int baseline;

    Vector s;
  
    // Initialize the font
    cvInitFont(&font, CV_FONT_VECTOR0, 0.5, 0.5, 0.0, 1.0);
    cvGetTextSize("FOOg", &font, &fontSize, &baseline);

    // Initialize some tracking stuff
    frame = avi.getFrame(0);
    frameSize = cvSize(frame->width, frame->height);
    //prevState = initStates[0];

    // Open output video
    outputAVI.open(fname, 10, frameSize);

	cvNamedWindow(UPF_WINDOW_NAME, CV_WINDOW_AUTOSIZE);
    cvShowImage(UPF_WINDOW_NAME, frame);
	cvReleaseImage(&frame);

	int j;
/*    // Draw the initial states for all the samples. Each sample per frame
	int i;
    for (i = 0; i < m_global.nInitStates; i++) {

		for (j = 0; j < m_global.nsamples; j++) {
			frame = avi.getFrame(i);
        m_iteration.kt_new[j]->setState(m_global.initStates[j][i]);

        m_iteration.kt_new[j]->drawLines(frame, CV_RGB(255, 255, 255));

		sprintf(text, "init state %d nsamples %d", i, j);

		cvPutText(frame, text, cvPoint(20, 20), &font, 
                  CV_RGB(255, 255, 255));

		outputAVI.append(frame);
		cvShowImage(UPF_WINDOW_NAME, frame);
		cvReleaseImage(&frame);

		}
    }
*/	
	
    frameIdx = m_global.nInitStates - 1;
//	prevState = m_global.initStates[0][frameIdx];

    // Draw the output of state x0 -- note that estimateX0 should have been
    // previously called by readyToTrack().

	// x0 is just the initStates[latest]

/*    for (j = 0; j < m_global.nsamples; j++) {
		frame = avi.getFrame(frameIdx);
		prevState = m_global.initStates[j][frameIdx];
		s = toFullState(m_iteration.dyn_new[j]->observe(), prevState);
		m_iteration.kt_new[j]->setState(s);
		m_iteration.kt_new[j]->drawLines(frame, CV_RGB(255, 0, 0));
		sprintf(text, "x0, sample %d", j);
		cvPutText(frame, text, cvPoint(20, 20), &font, CV_RGB(255, 0, 0));
		outputAVI.append(frame);
		cvShowImage(UPF_WINDOW_NAME, frame);
		cvReleaseImage(&frame);
	}
*/

    
    // Set model to initial (hand-set) state
	for (j = 0; j < m_global.nsamples; j++)
		m_iteration.kt_new[j]->setState(m_global.initStates[j][frameIdx]);

    // Initialize sample weights
	initialize_sample_weights();

	// initialize naccept and nreject
	naccept.clear();
	nreject.clear();
	
	// Tracking loop
	clock_t start, finish;
	double  duration;
	start = clock();
    for ( ; frameIdx < avi.getLength(); frameIdx++) {
		
		processFrame();
        
        // save history
        history.push_back(UPFHistoryItem(frameIdx, tracker, jointCtrs, trackerScore, UPFHistoryItem::FINAL));

		// draw result
//		drawAllSamples();
		drawResult();
    }
	finish = clock();
	duration = (double)(finish - start) / CLOCKS_PER_SEC;
	cout << "duration time = " << duration << endl;
	
    outputAVI.close();

	// save sample_move information
/*	ofstream file("samplemoveinfo.txt");
	saveSampleMoveInfo(file);
	file.close();*/

/*	ofstream file("tracker.txt");
	saveHistory(file);
	file.close();*/

}

/** run tracker for n runs, recording tracker results. */
void UPF::batchrun()
{
	int NRun = 5;
	int n, i, j;
	int ix, jx, kx, lx;  // starting position of filename
	char trackerfile[100], mcfile[100], jointfile[100], scorefile[100];

	initialize_sample_weights();
	vector<Vector> ys;
	ys.clear();

	if (m_global.opti == 0 && m_global.mc_step == 0) {
		ix = 0;
		jx = sprintf(trackerfile, "..\\sample projects\\results\\UPF_");
		kx = sprintf(jointfile, "..\\sample projects\\results\\jointCtr_UPF_");
		lx = sprintf(scorefile, "..\\sample projects\\results\\score_UPF_");
	}
	else if (m_global.opti == 0 && m_global.mc_step == 1) {
		ix = sprintf(mcfile, "..\\sample projects\\results\\mcinfo_UPF_");
		jx = sprintf(trackerfile, "..\\sample projects\\results\\UPF_mc_");
		kx = sprintf(jointfile, "..\\sample projects\\results\\jointCtr_UPF_mc_");
		lx = sprintf(scorefile, "..\\sample projects\\results\\score_UPF_mc_");
	}
	else if (m_global.opti == 1 && m_global.mc_step == 0 && m_global.NeffFlag == 0) {
		ix = 0;
		jx = sprintf(trackerfile, "..\\sample projects\\results\\OUPF_");
		kx = sprintf(jointfile, "..\\sample projects\\results\\jointCtr_OUPF_");
		lx = sprintf(scorefile, "..\\sample projects\\results\\score_OUPF_");
	}
	else if (m_global.opti == 1 && m_global.mc_step == 1) {
		ix = sprintf(mcfile, "..\\sample projects\\results\\mcinfo_OUPF_");
		jx = sprintf(trackerfile, "..\\sample projects\\results\\OUPF_mc_");
		kx = sprintf(jointfile, "..\\sample projects\\results\\jointCtr_OUPF_mc_");
		lx = sprintf(scorefile, "..\\sample projects\\results\\score_OUPF_mc_");
	}
	else if (m_global.opti == 1 && m_global.mc_step == 0 && m_global.NeffFlag == 1) {
		ix = 0; // mc_step == 0
		jx = sprintf(trackerfile, "..\\sample projects\\results\\OUPF_neff_");
		kx = sprintf(jointfile, "..\\sample projects\\results\\jointCtr_OUPF_neff_");
		lx = sprintf(scorefile, "..\\sample projects\\results\\score_OUPF_neff_");
	}

	for (n = 0; n < NRun; n++) {
		sprintf(trackerfile + jx, "Run_%d.txt", n); 
		sprintf(jointfile + kx, "Run_%d.txt", n);
		sprintf(mcfile + ix, "Run_%d.txt", n);
		sprintf(scorefile + lx, "Run_%d.txt", n);

		ofstream ftracker(trackerfile);
		ofstream fmcinfo(mcfile);
		ofstream fjoint(jointfile);
		ofstream fscore(scorefile);
		if (!ftracker.is_open() || !fmcinfo.is_open() || !fjoint.is_open() || !fscore.is_open()) {
			cout << "Error: can not open file to write!" << endl;
			return;
		}
		
		// Everything is ok; do initialization
    
		// Initialize history
		history.erase(history.begin(), history.end());
		frameIdx = m_global.nInitStates - 1;

		// Initialize dynamic model hidden state
//		vector<Vector> ys;
		for (j = 0; j < m_global.nsamples; j++) {
		for (i = 0; i < m_global.nInitStates; i++) {
			ys.push_back(fromFullState(m_global.initStates[j][i]));
		}
		m_iteration.dyn_new[j]->estimateX0(ys);
		m_iteration.dyn_old[j]->estimateX0(ys);
	
		ys.clear();
   		}
		
		// run the UPF tracker
//		frameIdx = m_global.nInitStates - 1;

		// Set model to initial (hand-set) state
		for (j = 0; j < m_global.nsamples; j++) {
			m_iteration.kt_new[j]->setState(m_global.initStates[j][frameIdx]);
			m_iteration.kt_old[j]->setState(m_global.initStates[j][frameIdx]);
		}

		m_iteration.p_new.clear();
		m_iteration.p_old.clear();

		// Initialize sample weights
		set_up_prior_conditions();
		
	
		// Tracking loop
		for ( ; frameIdx < avi.getLength(); frameIdx++) {
		
			processFrame();
        
			// save history
			history.push_back(UPFHistoryItem(frameIdx, tracker, jointCtrs, trackerScore, UPFHistoryItem::FINAL));
		}
	
		saveSampleMoveInfo(fmcinfo);
		saveHistory(ftracker);
		saveJointCtrs(fjoint);
		saveTrackerScore(fscore);
		fmcinfo.close();
		ftracker.close();
		fjoint.close();
		fscore.close();

		naccept.clear();
		nreject.clear();

	}
}


/**
    Hides the output window.
 */
void UPF::hide() {
    cvDestroyWindow(UPF_WINDOW_NAME);
}

/**
   Writes the final state estimate for each frame to the specified file.
   @param fout output filestream
 */
void UPF::saveHistory(ofstream &fout) {
    Vector s;
    
    vector<UPFHistoryItem>::iterator itr = history.begin();
    for ( ; itr != history.end(); itr++) {
        if (itr->type == UPFHistoryItem::FINAL) {
            s = itr->state;
            for (int i = 0; i < s.len(); i++) {
                fout << s.get(i) << " ";
            }
            fout << endl;
        }
    }
}

void UPF::saveJointCtrs(ofstream &fout)
{
	vector<CvPoint> jointCtrs;

	vector<UPFHistoryItem>::iterator itr = history.begin();
    for ( ; itr != history.end(); itr++) {
        if (itr->type == UPFHistoryItem::FINAL) {
            jointCtrs = itr->jointCtrs;
            for (int i = 0; i < jointCtrs.size(); i++) {
                fout << jointCtrs[i].x << " " << jointCtrs[i].y << " ";
            }
            fout << endl;
        }
    }
}

void UPF::saveTrackerScore(ofstream &fout)
{
	vector<UPFHistoryItem>::iterator itr = history.begin();
    for ( ; itr != history.end(); itr++) {
        if (itr->type == UPFHistoryItem::FINAL) {
            fout << itr->score << " " << endl;            
        }
    }
}

void UPF::saveSampleMoveInfo(ofstream &fout)
{
//	ofstream fout("..\\sample projects\\sample_move.txt");
	if (!fout.is_open()) {
		cout << "error" << endl;
		return;
	}
	int len = naccept.size();
	
	for (int i = 0; i < len; i++) {
		fout << naccept[i] << " " << nreject[i] << endl;
	}
}

/**
   Converts the short state used by the dynamics class (which may not
   include lengths or root position, or might use relative instead
   of absolute root position) to the full state used by the kinematic
   model.
    @param v shortened state vector
    @param extra a full state vector from which the missing parts of
        v are taken
    @return the new full state vector
 */
Vector UPF::toFullState(Vector v, Vector extra) {
    return v;

    // In this case, add root position and lengths
/*    Vector w(v.len() + 14);
    int j = 0;
    for (int i = 0; i < w.len(); i++) {
        if (i%2 == 1 || i == 2) {
            w.set(i, extra.get(i));
        }
        else {
            w.set(i, v.get(j));
            j++;
        }
    }
    return w;
*/   
    // In this case, we need to add the root position
    /*
    Vector w(v.len() + 2);
    int j = 0;
    for (int i = 0; i < w.len(); i++) {
        if (i == 1 || i == 2) {
            w.set(i, extra.get(i));
        }
        else {
            w.set(i, v.get(j));
            j++;
        }
    }
    return w;
    */
}


/**
   Converts the full state used by the kinematic model to a different
   state used by the dynamic model.
   @param v full state vector
   @return the shortened state vector
 */
Vector UPF::fromFullState(Vector v) {
    return v;

    // In this case, we remove root position as well as lengths
/*    Vector w(v.len() - 14);
    int j = 0;
    for (int i = 0; i < v.len(); i++) {
        if (i%2 != 1 && i != 2) {
            w.set(j, v.get(i));
            j++;
        }
    }
  
	return w;
*/    /*
    // In this case, we simply remove the root position
    Vector w(v.len() - 2);
    
    int j = 0 ;
    for (int i = 0; i < v.len(); i++) {
        if (i != 1 && i != 2) {
            w.set(j, v.get(i));
            j++;
        }
    }
    return w;
    */
}


/**
    Writes information pertaining to this project to the specified
    file.
    @param fout output filestream
 */
/*void UPF::writeToFile(ofstream &fout) {
    fout << "% AVI filename" << endl;
    if (aviFilename == NULL) {
        fout << " " << endl;
    }
    else {
        fout << aviFilename << endl;
    }
    
    fout << "% Dynamics filename" << endl;
    if (dynFilename == NULL) {
        fout << " " << endl;
    }
    else {
        fout << dynFilename << endl;
    }

    //fout << "% Kinematic model filename" << endl;
    //if (ktFilename == NULL) {
    //    fout << " " << endl;
    //}
    //else {
    //    fout << ktFilename << endl;
    //}

    fout << "% Kinematic model" << endl;
    kt->writeToFile(fout);

    fout << "% initial states" << endl;
    fout << nInitStates << endl;
    for (int i = 0; i < nInitStates; i++) {
        fout << initStates[i].len() << endl;
        for (int j = 0; j < initStates[i].len(); j++) {
            fout << initStates[i].get(j) << " ";
        }
        fout << endl;
    }
    
    fout << "% appearance model" << endl;
    eam.writeToFile(fout);
}*/


/**
    Tries to load the project in the specified file.  
    @param fin input filestream
    @param msg if there is an error, an error message is returned here
    @return true on success; on failure, return false and an 
        error message in msg.
 */
bool UPF::readFromFile(ifstream &fin, char **msg, int& nsamples) {
    int bufLen = 512;
    char buf[512];

	// Get nsamples
	m_global.nsamples = nsamples;
    
    // Get aviFilename
    IOFun::readStringWithComments(fin, buf, bufLen);
    if (strcmp(" ", buf) != 0) {
        if (!openAVI(buf, msg)) {
            return false;
        }
    }

    // Get dynFilename
    IOFun::readStringWithComments(fin, buf, bufLen);
    if (strcmp(" ", buf) != 0) {
        if (!openDynamics(buf, msg)) {
            return false;
        }
    }
    
	int i,j;

    // Get kinematic model
	long filestart = fin.tellg();
	// kinematic tree for tracker
	try {
		m_global.kt = new KinematicTree(fin);
	} catch (TrackerException te) {
		*msg = new char[256];
        sprintf(*msg, "Could not load kinematic model: %s", te.what());
        return false;
	}
	// kinematic trees for all the samples
	for (j = 0; j < m_global.nsamples; j++)
	try {
		fin.seekg(filestart);
        m_iteration.kt_new.push_back(new KinematicTree(fin));
		fin.seekg(filestart);
		m_iteration.kt_old.push_back(new KinematicTree(fin));
		
    } catch (TrackerException te) {
        *msg = new char[256];
        sprintf(*msg, "Could not load kinematic model: %s", te.what());
        return false;
    }

    // Get initial states
    int n = IOFun::readIntWithComments(fin);

	int len;
	for (i = 0; i < m_global.nInitStates && i < n; i++) {
        len = IOFun::readIntWithComments(fin);
        m_global.initStates[0][i].initialize(len);
        for (int k = 0; k < len; k++) {
            m_global.initStates[0][i].set(k, IOFun::readDoubleWithComments(fin));
        }
        m_global.initted[0][i] = true;
    }

	// Initialize prior model for initiate states
	m_global.priorSigma.initialize(len);
	m_global.priorSigma.setZero();
	// read prior deviation of states from file
	ifstream fin_prior("..\\sample projects\\initDev_stevejjmpg_nofeet.txt"); // for dancing
//	ifstream fin_prior("..\\sample projects\\greenleg_initDev_Patch.txt");
	if (fin_prior.is_open())
	for (i = 0; i < len; i++)
		m_global.priorSigma.set(i, IOFun::readDoubleWithComments(fin_prior));
	else
		sprintf(*msg, "could not load prior deviation for init states!");
	fin_prior.close();

	for (j = 1; j < m_global.nsamples; j++) {
		for (i = 0; i < m_global.nInitStates && i < n; i++) {
//			m_global.initStates[j][i].duplicate(m_global.initStates[0][i]);
			m_global.initStates[j][i] = m_global.initStates[0][i] + m_global.priorSigma.vectorproduct(randn(len));
			m_global.initted[j][i] = m_global.initted[0][i];
		}
	}


    // Get appearance model
    m_global.eam.readFromFile(fin);

	m_global.reg.readFromFile(fin);

	fin.close();


    return true;
}

void UPF::scaledUT(int old_sample)
{
	int nx = m_iteration.dyn_old[old_sample]->observe().len();  // state dimension
	
	P = m_iteration.p_old[old_sample] * (nx + KAPPA);
		
	int i, j;

/*	TNT::Array2D<double> Pb(nx, nx);
	TNT::Array2D<double> L;
	for(i = 0; i < nx; i++)
		for (j = 0; j < nx; j++)
		{
			Pb[i][j] = P.get(i,j);
		}

	JAMA::Cholesky<double> chol(Pb);
	if (chol.is_spd()) 
		L = chol.getL();			
  	else
	{	cout << "factorization was not complete.\n";
		return;
	}
*/

	P.SVD(U, V, D);
	P.setIdentity();
	for (i = 0; i < nx; i++)
		P.set(i, i, sqrt(D.get(i, i)));  // now P = sqrt(D)
	V = U * P;

	// sigma points
	sigmaPts.clear();

	sigmaPts.push_back(m_iteration.dyn_old[old_sample]->observe());
	for (i = 0; i < nx; i++)
	{
		for (j = 0; j < nx; j++)
			tmp.set(j, V.get(j, i));  // ith col of V
		
		sigmaPts.push_back(sigmaPts[0] + tmp);
		sigmaPts.push_back(sigmaPts[0] - tmp);
	}

}

void UPF::propagate(int new_sample, int old_sample)
{	
	int i;
	double score;
	double obs;

	// initialize mx, my, PxPred, K, Py, Pxy
	mx.setZero();
	my = 0;
	Py = 0;
	Pxy.setZero();
	PxPred.setZero();

	sigmaObs.clear();

//	Matrix Uf, Vf, Df;

	// copy vel, x
	m_iteration.dyn_new[new_sample]->copyStates(m_iteration.dyn_old[old_sample]);

	pred = m_iteration.kt_old[old_sample]->getState();

	// optimize predicted sigma points
/*	if (m_global.opti == 1)
	{
		for (i = 0; i < sigmaPts.size(); i++)
		{
			m_iteration.dyn_new[new_sample]->setState(sigmaPts[i]);			
			m_iteration.kt_new[new_sample]->setState(toFullState(
				m_iteration.dyn_new[new_sample]->predict(), pred));
			m_global.reg.prepForRegistration(displayImg, m_global.eam, *(m_iteration.kt_new[new_sample]), score);
			while (!m_global.reg.oneIteration(m_global.eam, *(m_iteration.kt_new[new_sample]), score)) {
			}
			sigmaPts[i] = fromFullState(m_iteration.kt_new[new_sample]->getState());  // update sigma points
			sigmaObs.push_back(exp(- score / 10));                                    // observation
			mx += sigmaPts[i];
			my += sigmaObs[i];
		}
		mx = mx * sigmaWts[1] + sigmaPts[0] * (sigmaWts[0] - sigmaWts[1]);
		my = my * sigmaWts[1] + sigmaObs[0] * (sigmaWts[0] - sigmaWts[1]);
	}
	else
*/	{
		for (i = 0 ; i < sigmaPts.size(); i++) {
			m_iteration.dyn_new[new_sample]->setState(sigmaPts[i]);  // update x
			sigmaPts[i] = m_iteration.dyn_new[new_sample]->predict();
			// additive noise
//			sigmaPts[i] = m_iteration.dyn_new[new_sample]->predict()
//				+ fromFullState(m_global.priorSigma).vectorproduct(randn(mx.len()));
			// 2nd way to add noise
/*			m_iteration.kt_new[new_sample]->setState(toFullState(sigmaPts[i], pred));
			hessian = m_global.eam.getHessian((m_iteration.kt_new[new_sample]), displayImg);
			P = (*hessian).inv() * 10;
//			display(P);
			
			Uf.initialize(P.rows(), P.cols());
			Vf.initialize(P.rows(), P.cols());
			Df.initialize(P.rows(), P.cols());
			P.SVD(Uf, Vf, Df);
			P.setIdentity();
			for (int j = 0; j < Df.cols(); j++)
				P.set(j, j, sqrt(Df.get(j, j)));
			sigmaX = Uf * P;
			sigmaPts[i] = m_iteration.dyn_new[new_sample]->predict() 
				+ fromFullState(sigmaX * randn(sigmaX.cols()));
			delete hessian;
            // end of 2nd way to add noise
*/

			mx += sigmaPts[i];// * sigmaWts[i];
			
			m_iteration.kt_new[new_sample]->setState(toFullState(sigmaPts[i], pred));
			
//			m_global.reg.prepForRegistration(displayImg, m_global.eam, *(m_iteration.kt_new[new_sample]), score);
			score = m_global.eam.getScore(m_iteration.kt_new[new_sample]);
			sigmaObs.push_back(exp(- score/ step));
			my += sigmaObs[i];// * sigmaWts[i];
		}

		mx = mx * sigmaWts[1] + sigmaPts[0] * (sigmaWts[0] - sigmaWts[1]);
		my = my * sigmaWts[1] + sigmaObs[0] * (sigmaWts[0] - sigmaWts[1]);

	}

	// "true" observation, corresponding to mx
	m_iteration.kt_new[new_sample]->setState(toFullState(mx, pred));
/*	if (m_global.opti == 1)
	{
		m_global.reg.prepForRegistration(displayImg, m_global.eam, *(m_iteration.kt_new[new_sample]), score);
		while (!m_global.reg.oneIteration(m_global.eam, *(m_iteration.kt_new[new_sample]), score)) {
		}
		obs = exp(-score/10);
	}
	else
*/	{
//		m_global.reg.prepForRegistration(displayImg, m_global.eam, *(m_iteration.kt_new[new_sample]), score);
		score = m_global.eam.getScore(m_iteration.kt_new[new_sample]);
		obs = exp(-score/ step);  // observation
	}

	for (i = 0 ; i < sigmaPts.size(); i++) {
		sigmaPts[i] = sigmaPts[i] - mx;
		sigmaObs[i] = sigmaObs[i] - my;
	}

	// PxPred, Py, Pxy
	for (i = 0 ; i < sigmaPts.size(); i++) {
		// calcualte PxPred
		vector2matrix(sigmaPts[i], mdiff);
		PxPred = mdiff * mdiff.transpose() + PxPred;
		Py += sigmaObs[i] * sigmaObs[i];// * sigmaWts[i];
		Pxy = sigmaPts[i] * sigmaObs[i]  + Pxy;
	}
	vector2matrix(sigmaPts[0], mdiff);
	PxPred = PxPred * sigmaWts[1] + mdiff * (mdiff.transpose() * (sigmaWts[sigmaWts.size()-1] - sigmaWts[1]));
	Py = Py * sigmaWts[1] + sigmaObs[0]*sigmaObs[0] * (sigmaWts[sigmaWts.size()-1] - sigmaWts[1]);
	Pxy = Pxy * sigmaWts[1] + sigmaPts[0] * (sigmaObs[0] * (sigmaWts[sigmaWts.size()-1] - sigmaWts[1]));

	// update
	if (Py == 0)
		Py = Py + 0.0001;
	K = Pxy *(1 / Py);
	mx = mx + K * (obs - my);
	vector2matrix(K, mdiff);
	m_iteration.p_new[new_sample] = PxPred - (mdiff * Py) * mdiff.transpose();
	if (m_iteration.p_new[new_sample].det() == 0) {
		P.setIdentity();
		m_iteration.p_new[new_sample] = m_iteration.p_new[new_sample] + P * 0.0001;
	}


}

void UPF::UKF(int new_sample, int old_sample)
{
	scaledUT(old_sample);
	propagate(new_sample, old_sample);

	// sample x from gaussian N(mx, p_new)
	// then dyn_new is updated by x
	P = m_iteration.p_new[new_sample];
	int nx = P.cols();
	
	P.SVD(U, V, D);

	int i;
	P.setIdentity();
	for (i = 0; i < nx; i++)
		P.set(i, i, sqrt(D.get(i, i)));
	sigmaX = U * P;

	tmp = mx + sigmaX * randn(mx.len());// + m_global.priorSigma.vectorproduct(randn(mx.len()));

	m_iteration.dyn_new[new_sample]->copyStates(m_iteration.dyn_old[old_sample]);
	m_iteration.dyn_new[new_sample]->setState(tmp);

	// set kt
	pred = m_iteration.kt_old[old_sample]->getState();
	m_iteration.kt_new[new_sample]->setState(toFullState(tmp, pred));
}

void UPF::evaluate_weights(int new_sample, int old_sample)
{
	if (m_global.opti == 1)
		m_iteration.tmp_weights[new_sample] = exp(-m_iteration.curScore[new_sample] / 10);
	else {
/*		double prior, proposal, like;
		like = exp(-m_iteration.curScore[new_sample] / step);
		
		xnew = m_iteration.dyn_new[new_sample]->observe();
		xold = m_iteration.dyn_old[old_sample]->observe();

		tmp = xnew - xold;
		prior = exp(-0.5*((m_iteration.p_old[old_sample].inv() * tmp) * tmp)) / 
			sqrt(m_iteration.p_old[old_sample].det());

		tmp = xnew - mx;
		proposal = exp(-0.5 * ((m_iteration.p_new[new_sample].inv() * tmp) * tmp )) / sqrt(m_iteration.p_new[new_sample].det());

		m_iteration.tmp_weights[new_sample] = like * prior / proposal;*/
		m_iteration.tmp_weights[new_sample] = exp(-m_iteration.curScore[new_sample] / step);
	}

}

/** 
	Gaussian random vector
  */
Vector UPF::randn(int len)
{
	Vector a;
	a.initialize(len);

	for (int i = 0; i < len; i++)
		a.set(i, gaussian_random());
	return a;
}

/**
  Display vector in console. For debug
*/
void UPF::display(Vector &v)
{
	cout << "vector length = " << v.len() << endl;
	for(int i=0; i < v.len(); i++)
		cout << v.get(i) << " ";
	cout << endl;
}

void UPF::display(Matrix &m)
{
	for (int i=0; i < m.rows(); i++)
		for (int j=0; j < m.cols(); j++)
			if (j != m.cols() - 1)
				cout << m.get(i,j) << " " ;
			else
				cout << m.get(i,j) << endl;
}

/**
	Show all the data in dyn and kt. For debug
	*/
void UPF::showData()
{
	int j;
	
	for (j = 0; j < m_global.nsamples; j++)
	{
		cout << "........................" << endl;
		
		cout << "dyn_new  sample = " << j << endl;
		display(m_iteration.dyn_new[j]->observe());
		

		cout << "dyn_old  sample = " << j << endl;
		display(m_iteration.dyn_old[j]->observe());

		cout << "kt_new sample = " << j << endl;
		display(m_iteration.kt_new[j]->getState());

		cout << "kt_old sample = " << j << endl;
		display(m_iteration.kt_old[j]->getState());
	}
}