// Data types of PF
#ifndef UPFDataType_INCLUDED
#define UPFDataType_INCLUDED

#include "cv.h"

#include "Dynamics.h"
#include "EdgeAppearanceModel.h"
#include "PatchAppearanceModel.h"
#include "KinematicTree.h"
#include "Registration.h"
#include "TrackerException.h"
#include "Vector.h"
#include "Matrix.h"

typedef struct UPFGlobalData {

	int mc_step;

	int opti;

	int NeffFlag;
	
	int nsamples;
	
	/** Number of initial states used */
	int nInitStates;  

	/** Prior Covariance Matrix */
//	Matrix priorCov;

	/** Prior Deviation (assume the state componets are i.i.d.)*/
	Vector priorSigma;

	/** 
        Array of bools, one per sample per initial state; an element is true
        if the corresponding initial state has been set.
     */
	vector<bool*> initted;

	/** for each sample: The list of initial states; an array of length nInitStates */
    vector<Vector*> initStates;

  	/** The appearance model */
	PatchAppearanceModel eam;
	
    /** The instance of the Registration class being used */
    Registration reg;

	/** A kinametic tree to represent the tracker (not the samples) */
	KinematicTree *kt;  // called by OPF::drawResult()

} UPFGlobalData;

typedef struct UPFIterationData {

	/** The kinematic model being used */
	vector<KinematicTree*> kt_new, kt_old;
	
	/** The dynamic model */
    vector<Dynamics*> dyn_new, dyn_old;
    	
    /** The covariance matirx */
    vector<Matrix> p_old, p_new;
	
	double *sample_weights, *cumul_prob_array, largest_cumulative_prob;

	double *tmp_weights;

	/** Current value of the objective function */
	double *curScore;  // directly related to sample_weights
} UPFIterationData;

#endif  // ! def UPFDataType_INCLUDED