// utility.cpp : implementation of some utility routines
//

#ifndef _UTILITY_CPP
#define _UTILITY_CPP

//////////////////////////
//
// Include
//
//////////////////////////
#include <math.h>
#include <stdlib.h>

/* The following are some utility routines. */

/* This is the worst random number routine in the known universe, but
   I use it for portability. Feel free to replace it. */
double uniform_random(void)
{
  return (double) rand() / (double) RAND_MAX;
}

/* This Gaussian routine is stolen from Numerical Recipes and is their
   copyright. */

double gaussian_random(void)
{
  static int next_gaussian = 0;
  static double saved_gaussian_value;

  double fac, rsq, v1, v2;

  if (next_gaussian == 0) {
    do {
      v1 = 2.0*uniform_random()-1.0;
      v2 = 2.0*uniform_random()-1.0;
      rsq = v1*v1+v2*v2;
    } while (rsq >= 1.0 || rsq == 0.0);
    fac = sqrt(-2.0*log(rsq)/rsq);
    saved_gaussian_value=v1*fac;
    next_gaussian=1;
    return v2*fac;
  } else {
    next_gaussian=0;
    return saved_gaussian_value;
  }
}

double evaluate_gaussian(double val, double sigma)
{
  /* This private definition is used for portability */
  static const double Condense_PI = 3.14159265358979323846;

  return 1.0/(sqrt(2.0*Condense_PI) * sigma) *
    exp(-0.5 * (val*val / (sigma*sigma)));
}


/* Indexing 
   copied from "Numerical Recipes in C"
   Section 8.3 Indexing and Ranking
   pp. 249 */
void indexx(int n, float *arrin, int *indx)
{
	int l, j, ir, indxt, i;
	float q;

	for (j=0; j<n; j++) indx[j]=j;  // Initialize the index array with consecutive integers.
	if(n==1) return;
	l=(n>>1) + 1;     /* From here on, we just have Heapsort, but with indirect indexing
						through indx in all references to arrin. */
	ir = n;
	for (;;)
	{
		if(l>1)
			q = arrin[(indxt=indx[--l-1])];
		else{
			q=arrin[(indxt=indx[ir-1])];
			indx[ir-1] = indx[0];
			if(--ir == 1) {
				indx[0] = indxt;
				return;
			}
		}
		i=l;
		j=l << 1;
		while (j<=ir) {
			if (j < ir && arrin[indx[j-1]] < arrin[indx[j]]) j++;
			if (q < arrin[indx[j-1]]) {
				indx[i-1] = indx[j-1];
				j += (i=j);
			}
			else j=ir+1;
		}
		indx[i-1]=indxt;
	}
}

int compare (const void * a, const void * b)
{
	return (int)( *(double*)a - *(double*)b );
}


// divisor is required to be positive
double myfmod(double dividend, double divisor)
{
	if (dividend >= 0)
		return fmod(dividend, divisor);
	else {
		return fmod(dividend, divisor) + divisor;
/*		double newdividend;
		newdividend = fmod(dividend, divisor);
		newdividend += divisor;
		return fmod(newdividend, divisor);*/
	}
}
/* End of utility routines */
#endif