#include <vector>
#include <iostream>
#include <fstream>
using namespace std;

#include "ARMADynamics.h"
#include "IOFun.h"
#include "Matrix.h"
#include "Vector.h"

/* Copy states */
/*ARMADynamics::ARMADynamics(const ARMADynamics &dyn)
{
	A1 = dyn.A1;
	A2.duplicate(dyn.A2);
	C.duplicate(dyn.C);
	Cinv.duplicate(dyn.Cinv);
	D.duplicate(dyn.D);
	x1.duplicate(dyn.x1);
	x2.duplicate(dyn.x2);
	history = dyn.history;

}*/

void ARMADynamics::copyStates(Dynamics* dyn_old)
{
	copyStates((ARMADynamics*)dyn_old);
}

void ARMADynamics::copyStates(ARMADynamics* dyn_old)
{
	x1 = dyn_old->x1;
	x2 = dyn_old->x2;
	history = dyn_old->history;
}

void ARMADynamics::setState(Vector &ys)
{
	x1 = ys;
}
/*
   Returns the number of states this model needs to initialize
   itself.
 */
int ARMADynamics::getNInitStates() {
    return 2;
}

void ARMADynamics::estimateX0(vector<Vector> &ys) {
    assert(ys.size() >= 2);

    //x2 = Cinv * ys[0];
    //x1 = Cinv * ys[1];
    x2 = ys[0];
    x1 = ys[1];
}

Vector ARMADynamics::observe() {
    return C*x1;
}

Vector ARMADynamics::predict() {
    // Save old stuff
    history.push_back(x2);
    
    Vector newx = A1*x1 + A2*x2 + D;
    x2 = x1;
    x1 = newx;
    return C*x1;
}

void ARMADynamics::correct(Vector y) {
    // Save old stuff
    history.push_back(x2);
    
    //x1 = Cinv * y;
    x1 = y;
}

void ARMADynamics::back() {
    if (history.size() >= 1) {
        x1 = x2;
        x2 = history.back();
        history.pop_back();
    }
}

void ARMADynamics::readFromFile(ifstream &fin) {
    int r, c;
    int i, j;
    double d;

    // Read A1
    r = IOFun::readIntWithComments(fin);
    c = IOFun::readIntWithComments(fin);
    A1.initialize(r,c);
    for (i = 0; i < r; i++) {
        for (j = 0; j < c; j++) {
            d = IOFun::readDoubleWithComments(fin);
            A1.set(i, j, d);
        }
    }

    // Read A2
    r = IOFun::readIntWithComments(fin);
    c = IOFun::readIntWithComments(fin);
    A2.initialize(r,c);
    for (i = 0; i < r; i++) {
        for (j = 0; j < c; j++) {
            d = IOFun::readDoubleWithComments(fin);
            A2.set(i, j, d);
        }
    }

    // Read C
    r = IOFun::readIntWithComments(fin);
    c = IOFun::readIntWithComments(fin);
    C.initialize(r,c);
    for (i = 0; i < r; i++) {
        for (j = 0; j < c; j++) {
            d = IOFun::readDoubleWithComments(fin);
            C.set(i, j, d);
        }
    }

    Cinv = C.inv();

    // Read D
    r = IOFun::readIntWithComments(fin);
    D.initialize(r);
    for (i = 0; i < r; i++) {
        d = IOFun::readDoubleWithComments(fin);
        D.set(i, d);
    }
}