#ifndef ARMA_DYNAMICS_INCLUDED
#define ARMA_DYNAMICS_INCLUDED

#include <vector>
#include <iostream>
#include <fstream>
using namespace std;

#include "Dynamics.h"
#include "Matrix.h"
#include "Vector.h"

/** @addtogroup dyn_group */
/* @{ */
/**
    Second-order ARMA model.
 */
class ARMADynamics : public Dynamics {
private:
    /** Plant and observation matrices */
    Matrix A1, A2, C, Cinv;
    /** State shift */
    Vector D;
    /** Past two hidden states */
    Vector x1, x2;
    /** History of hidden states */
    vector<Vector> history;

public:
//	ARMADynamics() {};
//	ARMADynamics(const ARMADynamics &dyn);

	virtual void copyStates(Dynamics* dyn_old);
	void copyStates(ARMADynamics* dyn_old);

	void setState(Vector &ys);

    int getNInitStates();
    void estimateX0(vector<Vector> &ys);
    
    Vector observe();
    Vector predict();
    void correct(Vector y);
    void back();

    void readFromFile(ifstream &fin);
};
/* @} */


#endif // !def ARMA_DYNAMICS_INCLUDED