#include "AVIViewer.h"
#include "cv.h"
#include "highgui.h"

#include <iostream>
#include <fstream>
using namespace std;

AVIViewer::AVIViewer() {
    seqOpen = false;
}

/**
    Constructor.  Opens an AVI file.  
    @param fname filename
 */
AVIViewer::AVIViewer(const char * fname) {
    seqOpen = false;
    open(fname);
}

/**
    Destructor.  Releases the AVI file.
 */
AVIViewer::~AVIViewer() {
    if (seqOpen) {
        cvReleaseCapture(&aviFile);
    }
}

/**
    Opens an AVI file.
    @param fname filename
 */
bool AVIViewer::open(const char * fname) {
    if (seqOpen) {
        cvReleaseCapture(&aviFile);
    }

    aviFile = cvCaptureFromAVI(fname);
    seqOpen = true;
    seqLength = (int)cvGetCaptureProperty(aviFile, CV_CAP_PROP_FRAME_COUNT);
    
    return true;
}

/**
    Gets a frame from the AVI.
    @param frame zero-based index of frame to retrieve
    @return the requested video frame, as an ipl image
 */
IplImage * AVIViewer::getFrame(int frame) {
    if (frame < seqLength) {
        cvSetCaptureProperty(aviFile, CV_CAP_PROP_POS_FRAMES, frame);
        IplImage *aviImg = cvQueryFrame(aviFile);
        IplImage *img = cvCreateImage(cvSize(aviImg->width, aviImg->height),
                                      aviImg->depth, aviImg->nChannels);
        cvCopy(aviImg, img);
        img->origin = IPL_ORIGIN_BL;
        return img;
    }
    else {
        return NULL;
    }
}

/**
    Gets a grayscale version of a frame from the AVI.
    @param frame zero-based index of frame to retrieve
    @return the requested video frame in grayscale, as an ipl image
 */
IplImage * AVIViewer::getGrayFrame(int frame) {
    IplImage *img = getFrame(frame);
    if (img != NULL) {
        IplImage *grayImg = cvCreateImage(cvSize(img->width, img->height), 
                                          IPL_DEPTH_8U, 1);
        grayImg->origin = IPL_ORIGIN_BL;
        cvConvertImage(img, grayImg, 0);
        cvReleaseImage(&img);
        return grayImg;
    }
    else {
        return NULL;
    }
}
