#include "AVIWriter.h"
#include "cv.h"
#include "highgui.h"

AVIWriter::AVIWriter() {
    aviFile = NULL;
    nextFrameIdx = 0;
}

/**
    Constructor.  Opens an AVI file for writing.
    @param fname filename
    @param fps frames per second
    @param frameSize width and height of frames
 */
AVIWriter::AVIWriter(const char * fname, int fps, CvSize frameSize) {
    aviFile = NULL;
    nextFrameIdx = 0;
    open(fname, fps, frameSize);
}

/**
    Destructor.  Closes AVI file.
 */
AVIWriter::~AVIWriter() {
    if (streamOpen()) {
        close();
    }
}

/**
    Opens an AVI file for writing and associates it with this
    object.  Currently, this function creates uncompressed AVI's.
    To have a dialog appear at runtime where the user can choose
    the compression, uncomment the first call to cvCreateAVIWriter
    and comment out the second call.
    @param fname filename
    @param fps frames per second
    @param frameSize frame width and height
    @return true on success, false otherwise
 */
bool AVIWriter::open(const char * fname, int fps, CvSize frameSize) {
    if (streamOpen()) {
        close();
    }

    nextFrameIdx = 0;
    //aviFile = cvCreateAVIWriter(fname, -1, fps, frameSize);   // -1 -> choose compression from dialog
    aviFile = cvCreateAVIWriter(fname, 541215044, fps, frameSize); // 541215044 -> uncompressed

    return streamOpen();
}

/**
    Closes the current AVI file.
    @return true on success
 */
bool AVIWriter::close() {
    if (!streamOpen()) {
        return true;
    }

    cvReleaseVideoWriter(&aviFile);
    aviFile = NULL;

	return true;
}

/**
    Adds the image to the end of the AVI file.
    @param img image
    @return true on success, false if img is NULL
 */
bool AVIWriter::append(IplImage *img) {
    if (img != NULL) {
        cvWriteToAVI(aviFile, img);
        nextFrameIdx++;
        return true;
    }
    return false;
}

/**
    Converts the CvMat to an ipl image and appends that image
    to the AVI file.
    @param mat the matrix
    @return true on success.
 */
bool AVIWriter::append(CvMat *mat) {
    CvSize size;
    size.width = mat->cols;
    size.height = mat->rows;

    IplImage *img = cvCreateImage(size, IPL_DEPTH_8U, 1);
    cvConvertScale(mat, img);

    if (img != NULL) {
        IplImage *rgbImg = cvCreateImage(size, IPL_DEPTH_8U, 3);
        cvConvertImage(img, rgbImg);
        cvWriteToAVI(aviFile, rgbImg);
        nextFrameIdx++;
        cvReleaseImage(&rgbImg);
        cvReleaseImage(&img);
        return true;
    }
    return false;
}
