#ifndef AVI_WRITER_INCLUDED
#define AVI_WRITER_INCLUDED

#include "cv.h"
#include "highgui.h"

/** @addtogroup util_group */
/* @{ */
/**
    Allows user to insert ipl images into an AVI file.
    This is a wrapper for some OpenCV calls.
    Theoretically, I believe it should be able
    to create any kind of AVI file you have the codec
    for.  However, I've only been able to get it
    to work with uncompressed files.
    
    Currently, the open() function is set up to automatically
    create an uncompressed file.  See the documentation for
    the open() function for information on how to have the
    program bring up the "choose codec" dialog so the user
    can choose the compression to use at run time.
 */
class AVIWriter {
public:
    AVIWriter();
    AVIWriter(const char * fname, int fps, CvSize frameSize);
    ~AVIWriter();
    bool open(const char * fname, int fps, CvSize frameSize);
    bool close();
    bool append(IplImage *img);
    bool append(CvMat *mat);
    /**
        @return zero-based index of next frame to be added
     */
    int getNextFrameIdx() { return nextFrameIdx; }

private:
    CvVideoWriter *aviFile;
    int nextFrameIdx;

    bool streamOpen() { return aviFile != NULL; }
};
/* @} */

#endif // !defined AVI_WRITER_INCLUDED