#include <iostream>
#include <fstream>
#include <vector>
using namespace std;

#include "ConstVelDynamics.h"
#include "Vector.h"

/*
	Copy constructor
 */
/*ConstVelDynamics::ConstVelDynamics(ConstVelDynamics &dyn) {
	history = dyn.history;
	lastx = dyn.lastx;
	vel = dyn.vel;
	x = dyn.x;
}*/


void ConstVelDynamics::copyStates(Dynamics* dyn_old)
{
	copyStates((ConstVelDynamics*)dyn_old);
/*	dyn_old = (ConstVelDynamics*)dyn_old;
	history = dyn_old.history;
	lastx = dyn_old.lastx;
	vel = dyn_old.vel;
	x = dyn_old.x;*/
}

void ConstVelDynamics::copyStates(ConstVelDynamics* dyn_old)
{
	history = dyn_old->history;
	lastx = dyn_old->lastx;
	vel = dyn_old->vel;
	x = dyn_old->x;
}

void ConstVelDynamics::setState(Vector &ys)
{
	x = ys;
}

/*
   Return the number of states needed to initialize this model.
 */
int ConstVelDynamics::getNInitStates() {
    return NUM_INIT_STATES;
}

/*
   ys should contain the first two states from the motion.
   This function sets the current velocity to the difference
   between these two states, and sets the current state to 
   the second of these two states.
 */
void ConstVelDynamics::estimateX0(vector<Vector> &ys) {
    assert(ys.size() >= NUM_INIT_STATES);

    Vector y1, y2;
    y1 = ys[0];
    y2 = ys[1];

    x = lastx = y2;
    vel = y2 - y1;
}

/*
   Return the current state.
 */
Vector ConstVelDynamics::observe() {
    return x;
}

/*
   Predict the next state as x + vel
 */
Vector ConstVelDynamics::predict() {
    // Save old stuff
    history.push_back(x);
    history.push_back(lastx);
    history.push_back(vel);
    
    x = x + vel; 
    return x;
}

/*
   Re-estimate velocity as y - current state.  Set the current
   state to y.
 */
void ConstVelDynamics::correct(Vector y) {
    // Save old stuff
    history.push_back(x);
    history.push_back(lastx);
    history.push_back(vel);
    
    vel = y - lastx;
    x = lastx = y;
}

/* Resets x, lastx, vel to previous values */
void ConstVelDynamics::back() {
    if (history.size() >= 3) {
        vel = history.back();
        history.pop_back();
        
        lastx = history.back();
        history.pop_back();
        
        x = history.back();
        history.pop_back();
    }
}

/*
   Does nothing.
 */
void ConstVelDynamics::readFromFile(ifstream &fin) { }