#ifndef DYNAMICS_INCLUDED
#define DYNAMICS_INCLUDED

#include <iostream>
#include <fstream>
#include <vector>
using namespace std;

#include "Vector.h"

/** @defgroup dyn_group Dynamics Model */
/* @{ */
/** Abstract base class of all dynamic models */
class Dynamics {
public:

//	Dynamics() {};

    /** Copy Constructor */
//	Dynamics(const Dynamics &dyn);

	/** Copy States
		Assign states the value of states in dyn_old to */
	virtual void copyStates(Dynamics* dyn_old) = 0;

	virtual void setState(Vector &ys) = 0;
	
	/** Returns the number of states needed to initialize this model */
    virtual int getNInitStates() = 0;

    /** 
        Estimates the initial hidden state, given the initial states in ys.
        @param ys a vector of Vector objects.  Each one is an initial state
     */
    virtual void estimateX0(vector<Vector> &ys) = 0;
    
    /** Returns output generated by current hidden state estimate */
    virtual Vector observe() = 0;

    /** Makes a prediction about the next hiddent state; updates hidden
       state estimate. */
    virtual Vector predict() = 0;

    /** Corrects the hidden state estimate based on the observation y */
    virtual void correct(Vector y) = 0;
    
    /** Resets the hidden state(s) to a previous value(s) */
    virtual void back() = 0;

    /** Loads the parameters for this model from the specified file */
    virtual void readFromFile(ifstream &fin) = 0;
};
/* @} */


#endif // !def DYNAMICS_INCLUDED