#include <iostream>
#include <fstream>
#include <limits>
#include <cstring>
using namespace std;

#include "IOFun.h"

/**
    Reads the next "token" from a text file that may contain 
    comments that start with '%'. If
    the next token won't fit in 'token', you get the part that will fit, and
    the rest of it comes next time.
    If there is no next token, the first byte of 'token' will be set to '\\0'.
    This should indicate end of file.
    @param fin filestream
    @param token string is returned here; should already be 
        allocated
    @param maxTokenLen size of space allocated for 'token';
        at most this many characters will be returned
    @param delims list of possible delimiting characters between
        tokens
 */
void IOFun::readTokenWithComments(ifstream &fin, char *token, 
                                  int maxTokenLen, const char *delims) {
    // OK, I'm sure what I'm about to do here is not an efficient way to do
    // this.  But it should work at least, and that's what counts.
    char c;
    bool inToken = false;
    int t = 0;

    while (fin.get(c)) {
        if (c == COMMENT_CHAR) {
            // The rest of this line is a comment.  Throw it away.
			fin.ignore(numeric_limits<int>::max(), '\n');
        }
        else if (strchr(delims, c)) {
            // This character is a delimiter
            if (inToken) {
                break;
            }
        }
        else {
            // This character must be part of a token!
            inToken = true;
            token[t++] = c;
            if (t == maxTokenLen - 1) {
                break;
            }
        }
    }
    token[t] = '\0';
}

/**
   Reads a double from a file which may contain comments
   that start with '%'.
   @param fin filestream
   @return next double in the text file
 */
double IOFun::readDoubleWithComments(ifstream &fin) {
	double d;
	char next = fin.peek();
	while (next == COMMENT_CHAR || next == ' ' || next == '\n') {
		if (next == COMMENT_CHAR) {
			fin.ignore(numeric_limits<int>::max(), '\n');
        }
        else {
            fin.ignore();
        }
        next = fin.peek();
	}
	fin >> d;
	return d;
}

/**
   Reads an int from a file which may contain comments
   that start with '%'
   @param fin filestream
   @return next int in the text file
 */
int IOFun::readIntWithComments(ifstream &fin) {
	int i;
	char next = fin.peek();
	while (next == COMMENT_CHAR || next == ' ' || next == '\n') {
		if (next == COMMENT_CHAR) {
			fin.ignore(numeric_limits<int>::max(), '\n');
        }
        else {
            fin.ignore();
        }
        next = fin.peek();
	}
	fin >> i;
	return i;
}

/**
   Reads a string from a file which may contain comments
   that start with '%'.  Returns the first line which does
   not start with '%', up to any '%' on that line.  If next line
   is longer than 'n' characters, returns first 'n' characters
   of the line.  The rest should be returned next time.
   @param fin filestream
   @param buf buffer in which to return the line
   @param n length of buffer; at most this many characters are returned
        in buf
 */
void IOFun::readStringWithComments(ifstream &fin, char *buf, int n) {
    char *loc = (char*)1;
    buf[0] = '\0';
    while (loc != NULL && fin.getline(buf, n)) {
        loc = strchr(buf, COMMENT_CHAR);
    }
}
