#ifndef KINEMATICTREE_INCLUDED
#define KINEMATICTREE_INCLUDED

#include <iostream>
#include <fstream>
#include <vector>
using namespace std;

#include "cv.h"
#include "SPMLink.h"
#include "Vector.h"
#include "Matrix.h"


/** @defgroup kin_group Kinematic Model */
/* @{ */
/** Kinematic model */
class KinematicTree {
private:
    /** Root link in the kinematic tree */
	SPMLink root;

    /** Number of states in the kinematic state vector */
	int nStates;

    /** Current state vector */
	Vector savedState;

    /** List of all the links in the tree */
	vector<SPMLink*> links;

public:
	KinematicTree();
//	KinematicTree(const KinematicTree &kt);
	KinematicTree(ifstream &fin);
	
	const SPMLink * operator[](const int i) const;

	bool setState(const Vector &v);
	Vector getState();
	int getStateLen();
    int countLinks();
    
	void getControlPts(vector<CvPoint> &controlPts);
	void setStateFromControlPts(CvPoint controlPt, int cidx);

	void getJointCenters(vector<CvPoint> &jointCenters);

    void writeToFile(ofstream &fout);

	void drawFilled(IplImage *image, CvScalar color);
	void drawBoxes(IplImage *image, CvScalar color);
	void drawLines(IplImage *image, CvScalar color);
    void drawSticksAndBalls(IplImage *image, CvScalar color, int r);
};
/* @} */



#endif // !def KINEMATICTREE_INCLUDED
