#include <cmath>

#include "MathFun.h"
#include "cv.h"
#include "Vector.h"
#include "Matrix.h"

/**
    Calculates the distance between two 2-d points.
    @param p1 point 1
    @param p2 point 2
    @return distance
 */
double MathFun::dist(const CvPoint &p1, const CvPoint &p2) {
    double dx = p1.x - p2.x;
    double dy = p1.y - p2.y;

    return sqrt(dx*dx + dy*dy);
}

/**
    Rounds a number.
    @param d number
    @return rounded number
 */
int MathFun::round(double d) {
    return fabs(floor(d) - d) < fabs(ceil(d) - d) 
           ? (int)floor(d) : (int)ceil(d);
}

/**
   Calculates
        vout = m * v.
   vout should already be dimensioned properly.
   @param m matrix
   @param v vector
   @param vout return value; should already be allocated
        and should have proper size
 */
void MathFun::matVecMul(const Matrix &m, const Vector &v, 
                                  Vector &vout) {
    assert(m.cols() == v.len() && vout.len() == m.rows());

    cvMatMul(m.matrix, v.vector, vout.vector);
}

/*
   Calculates
       MtM = Mt * transpose(Mt).
    MtM should already be dimensioned properly.
    @param Mt matrix
    @param MtM return value; should already be allocated
        and should have proper size.
 */
void MathFun::timesTranspose(const Matrix &Mt, Matrix &MtM) {
    assert(MtM.cols() == MtM.rows() && MtM.cols() == Mt.rows());

    int row, col, i;
    double acc;

    for (row = 0; row < MtM.rows(); row++) {
        for (col = row; col < MtM.cols(); col++) {
            acc = 0;
            for (i = 0; i < Mt.cols(); i++) {
                acc += Mt.get(row, i) * Mt.get(col, i);
            }
            MtM.set(row, col, acc);
            MtM.set(col, row, acc);
        }
    }
}
