#ifndef MATRIX_INCLUDED
#define MATRIX_INCLUDED

#include "cv.h"

class Vector;

/** @addtogroup util_group */
/* @{ */
/**
    A wrapper around an OpenCV CvMat object, with some
    useful functions.
 */
class Matrix {
public:
    Matrix();
    Matrix(int rows, int cols);
    Matrix(const Matrix &m);
    ~Matrix();

    void initialize(int rows, int cols);

    /**
        @return number of rows in the matrix
     */
    int rows() const { return height(); }
    /**
        @return number of columns in the matrix
     */
    int cols() const { return width(); }
    /**
        @return width (number of columns) of matrix
     */
    int width() const;
    /**
        @return height (number of rows) of matrix
     */
    int height() const;
    double operator()(int r, int c) const { return get(r, c); }
    double get(int r, int c) const;
    void set(int r, int c, double d);
    void setZero();
	void setIdentity();

    Matrix operator=(const Matrix &m);
    Vector operator*(const Vector &v) const;
    Matrix operator*(const Matrix &m);
	Matrix operator*(double dbl) const;
    Matrix operator+(const Matrix &m);
    Matrix operator-(const Matrix &m);

    Matrix inv();
    Matrix transpose();
	double det();

	// A = U' * D * V 
	// A = matrix
	void SVD(Matrix &U, Matrix &V, Matrix &D);

    CvMat *matrix;

private:
    void duplicate(const Matrix &m);
    void destroy();
};
/* @} */

#endif // !def MATRIX_INCLUDED