#ifndef MODEL_POINT_INCLUDED
#define MODEL_POINT_INCLUDED

/** @addtogroup util_group */
/* @{ */
/**
    Used by kinematic model to represent point locations.
    Used by some appearance models (EdgeAppearanceModel) to store information
    about points in the model
 */
class ModelPoint {
public:
    ModelPoint(double px, double py, double pd) : x(px), y(py), d(pd) { }
    ModelPoint(double px, double py) : x(px), y(py) { }
    ModelPoint(double px, double py, int pval) : x(px), y(py), value(pval) { }
    ModelPoint() { }
    
    bool operator==(const ModelPoint &rhs) const {
        return (x == rhs.x) && (y == rhs.y);
    }

    ModelPoint operator+(const ModelPoint &mp) const {
        ModelPoint retval;
        retval.x = x + mp.x;
        retval.y = y + mp.y;
        return retval;
    }
   
    /** point coordinates. */
    double x, y;  
    /** distance from the link origin to the projection of this 
        point onto the link axis */
    double d;      
	/** color intensity value of pixel */
    int value;     
};


/**
   This function object is used to compare two ModelPoints so they can be
   stored in a set.  This should not be called directly.
 */
struct ltmp {
    bool operator()(const ModelPoint &mp1, const ModelPoint &mp2) const {
        if (mp1.x == mp2.x) {
            return mp1.y < mp2.y;
        }
        else {
            return mp1.x < mp2.x;
        }
    }
};
/* @} */

#endif // !def MODEL_POINT_INCLUDED
