#include <iostream>
#include <fstream>
#include <vector>
using namespace std;

#include "NoDynamics.h"
#include "Vector.h"

/* Copy States */
/*NoDynamics::NoDynamics(const NoDynamics &dyn)
{
}*/

void NoDynamics::copyStates(Dynamics* dyn_old)
{
	copyStates((NoDynamics*)dyn_old);
}

void NoDynamics::copyStates(NoDynamics* dyn_old)
{
	x = dyn_old->x;
	history = dyn_old->history;
}

void NoDynamics::setState(Vector &ys)
{
	x = ys;
}

/*
   Return the number of states needed to initialize this model.
 */
int NoDynamics::getNInitStates() {
    return NUM_INIT_STATES;
}

/*
   ys should contain the first two states from the motion.
   This function sets the current velocity to the difference
   between these two states, and sets the current state to 
   the second of these two states.
 */
void NoDynamics::estimateX0(vector<Vector> &ys) {
    assert(ys.size() >= NUM_INIT_STATES);
    x = ys[0];
}

/*
   Return the current state.
 */
Vector NoDynamics::observe() {
    return x;
}

/*
   Predict the next state
 */
Vector NoDynamics::predict() {
    // Save old state
    history.push_back(x);

    return x;
}

void NoDynamics::correct(Vector y) {
    // Save old state
    history.push_back(x);

    x = y;
}

/*
   Resets the hidden state to a previous value.
 */
void NoDynamics::back() {
    if (history.size() >= 1) {
        x = history.back();
        history.pop_back();
    }
}

/*
   Does nothing.
 */
void NoDynamics::readFromFile(ifstream &fin) { }