#ifndef REGISTRATION_INCLUDED
#define REGISTRATION_INCLUDED

#include <iostream>
#include <fstream>
using namespace std;

#include "cv.h"

#include "AppearanceModel.h"
#include "KinematicTree.h"
#include "Vector.h"

#include "..//Base Tracker//utility.h"

/** @defgroup reg_group Registration Module */
/* @{ */
/** Performs image registration */
class Registration {
private:
    /** Maximum number of iterations */
    enum {MAX_ITERS = 20};
    
    /** Tolerances used in test for termination*/
    double TREL, TABS;

    /** Current objective function value */
    double curScore;

    /** norm of gradient at beginning of registration */
    double normg0;

    /** Current norm of gradient */
    double normg;

    /** Number of iterations so far */
    int cnt;

    /** Kinematic state after linesearch */
    Vector stateNew;

    /** Optional bound constraints on state vector; NULL for none */
    Vector *lowerBounds, *upperBounds;

    /** Indicates whether or not corresponding constraints should be
        enforced */
    bool *enforceBounds;

    void getDescentDir(AppearanceModel &am, KinematicTree &kt, 
        Vector &descentDir, Vector &JtR, double &score);
    bool linesearch(AppearanceModel &am, KinematicTree &kt, 
        Vector &descentDir, double curf, double &newScore);
    bool goldenSearch(AppearanceModel &am, KinematicTree &kt, 
        Vector &descentDir, double initf, double &newScore);

public:
    /** Current local gradient estimate */
    Vector grad;
        
    /** Current estimated descent direction */
    Vector descentDir;

    Registration();
    
    // All at once interface
    void registerImage(IplImage *image, AppearanceModel &am,
        KinematicTree &kt, double &score);

    // Step-by-step interface
    void prepForRegistration(IplImage *image, AppearanceModel &am,
        KinematicTree &kt, double &score);
    bool oneIteration(AppearanceModel &am, KinematicTree &kt, double &score);

    // File I/O
    void writeToFile(ofstream &fout);
    void readFromFile(ifstream &fin);
};
/* @} */

#endif // !def REGISTRATION_INCLUDED