#include <vector>
using namespace std;

#include "SSIDDynamics.h"
#include "IOFun.h"
#include "Vector.h"

SSIDDynamics::SSIDDynamics() { }

/* Copy States */
/*SSIDDynamics::SSIDDynamics(const SSIDDynamics &dyn)
{
}*/

void SSIDDynamics::copyStates(Dynamics* dyn_old)
{
	copyStates((SSIDDynamics*)dyn_old);
}

void SSIDDynamics::copyStates(SSIDDynamics* dyn_old)
{
	P = dyn_old->P;
	x = dyn_old->x;
//	Phistory = dyn_old->Phistory;
//	xhistory = dyn_old->xhistory;
}

void SSIDDynamics::setState(Vector &ys)
{
	x = ys;
}

/*
   Return the number of states needed to initialize this model.
 */
int SSIDDynamics::getNInitStates() {
    return NUM_INIT_STATES;
}


/*
   Estimate the initial x_hat from a number of observations.
   Initialize other matrices.
 */
void SSIDDynamics::estimateX0(vector<Vector> &ys) {
    assert(ys.size() >= NUM_INIT_STATES);

    /*
    Vector y0 = removeLengths(ys.front());
    
    x = Cinv*y0;
    */
    
    /*********************/
    // A hack:
    ifstream fin("x.txt");
    int l;
    double d;
    
    fin >> l;
    x.initialize(l);

    for (int i = 0; i < l; i++) {
        fin >> d;
        x.set(i, d);
    }

    fin.close();


    /*********************/

    P.initialize(A.rows(), A.cols());
    P.setZero();
    for (i = 0; i < P.rows(); i++) {
        P.set(i, i, 1.0);
    }
}

/*
   Based on the current estimate for x, predict the output.
 */
Vector SSIDDynamics::observe() {
    return C*x;
}

/*
   Do a time update.
 */
Vector SSIDDynamics::predict() {
    // Save old stuff
    xhistory.push_back(x);
    Phistory.push_back(P);
        
    x = A*x;
    P = A*P*A.transpose() + Q;
    return x;
}

/*
   Integrate an observation into the estimate for x.
 */
void SSIDDynamics::correct(Vector y) {
    // Save old stuff
    xhistory.push_back(x);
    Phistory.push_back(P);
        
    Matrix K = P*C.transpose()*(C*P*C.transpose() + R).inv();
    x = x + K*(y - C*x);
    P = P - K*C*P;
}


/*
   Resets P and x to previous values.
 */
void SSIDDynamics::back() {
    if (xhistory.size() >= 1 && Phistory.size() >= 1) {
        x = xhistory.back();
        xhistory.pop_back();
        
        P = Phistory.back();
        Phistory.pop_back();
    }
}

void SSIDDynamics::readFromFile(ifstream &fin) {
    int r, c;
    int i, j;
    double d;

    // Read A
    r = IOFun::readIntWithComments(fin);
    c = IOFun::readIntWithComments(fin);
    A.initialize(r,c);
    for (i = 0; i < r; i++) {
        for (j = 0; j < c; j++) {
            d = IOFun::readDoubleWithComments(fin);
            A.set(i, j, d);
        }
    }

    // Read C
    r = IOFun::readIntWithComments(fin);
    c = IOFun::readIntWithComments(fin);
    C.initialize(r,c);
    for (i = 0; i < r; i++) {
        for (j = 0; j < c; j++) {
            d = IOFun::readDoubleWithComments(fin);
            C.set(i, j, d);
        }
    }

    Cinv = C.inv();

    // Read Q
    r = IOFun::readIntWithComments(fin);
    c = IOFun::readIntWithComments(fin);
    Q.initialize(r,c);
    for (i = 0; i < r; i++) {
        for (j = 0; j < c; j++) {
            d = IOFun::readDoubleWithComments(fin);
            Q.set(i, j, d);
        }
    }

    // Read R
    r = IOFun::readIntWithComments(fin);
    c = IOFun::readIntWithComments(fin);
    R.initialize(r,c);
    for (i = 0; i < r; i++) {
        for (j = 0; j < c; j++) {
            d = IOFun::readDoubleWithComments(fin);
            R.set(i, j, d);
        }
    }
}