#ifndef SSID_DYNAMICS_INCLUDED
#define SSID_DYNAMICS_INCLUDED

#include "Dynamics.h"
#include "Matrix.h"
#include "Vector.h"

/** @addtogroup dyn_group */
/* @{ */
/** LDS dynamic model based on subspace identification */
class SSIDDynamics : public Dynamics {
private:
    /** Plant matrix */
    Matrix A;

    /** Observation matrix */
    Matrix C, Cinv;
    
    /** Noise covariances */
    Matrix Q, R;

    /** Error covariance estimate */
    Matrix P;
    
    /**  Current state estimate, either before or after observation */
    Vector x; 

    /** History of hidden state estimates */
    vector<Vector> xhistory;

    /** History of error covariance estimates */
    vector<Matrix> Phistory;

    /** To avoid problems, this must be at least 1 */
    enum {NUM_INIT_STATES = 1};

public:

//	SSIDDynamics(const SSIDDynamics &dyn);

	virtual void copyStates(Dynamics* dyn_old);
	void copyStates(SSIDDynamics* dyn_old);

	void setState(Vector &ys);

    int getNInitStates();
    SSIDDynamics();

    void estimateX0(vector<Vector> &ys);
    
    Vector observe(); 
    Vector predict();
    void correct(Vector y);
    void back();

    void readFromFile(ifstream &fin);
};
/* @} */



#endif // !def SSID_DYNAMICS_INCLUDED