#ifndef VECTOR_INCLUDED
#define VECTOR_INCLUDED

#include "cv.h"

class Matrix;

/** @addtogroup util_group */
/* @{ */
/**
    A wrapper around a one-dimensional CvMat object.
    It is implemented as a column vector -- not that
    that matters.
*/
class Vector {
public:
    Vector();
    Vector(int len);
    Vector(const Vector &v);
    ~Vector();

    void initialize(int len);

    int len() const;
    double operator()(int i) const { return get(i); }
    double get(int i) const;
    void set(int i, double d);
    void setZero();
    void negate();
    bool isZero();
	
	Vector vectorproduct(const Vector &v);
	
    Vector operator=(const Vector &v);
    Vector operator-();
    Vector operator+(const Vector &v) const;
    void operator+=(const Vector &v);
    Vector operator-(const Vector &v) const;
    double operator*(const Vector &v) const;
    Vector operator*(double s) const;
    
    friend bool operator==(const Vector& v1, const Vector& v2);
    friend bool operator!=(const Vector& v1, const Vector& v2);

    CvMat *vector;

//private:
    void duplicate(const Vector &v);
    void destroy();

};
/* @} */

bool operator==(const Vector& v1, const Vector& v2);
bool operator!=(const Vector& v1, const Vector& v2);

#endif // !def VECTOR_INCLUDED