#include "cv.h"
#include "highgui.h"

#include "EdgeThresholdSetter.h"
#include "ImgFun.h"

/**
   Constructor.  Initializes the threshold setter.  
   img should be a single-channel image.
   @param img the image to do an edge transform on
   @param lowThresh low threshold for edge detector
   @param highThresh high threshold for edge detector
 */
EdgeThresholdSetter::EdgeThresholdSetter(IplImage *img, 
                                         int lowThresh, int highThresh) {
    edgeImg = NULL;

    CvSize size = cvSize(img->width, img->height);
    origImg = ImgFun::newImage(size, IPL_DEPTH_8U, 1);
    cvCopy(img, origImg);
    setThresholds(lowThresh, highThresh);
}

/** 
    Destructor.  Hides output window and cleans up.
 */
EdgeThresholdSetter::~EdgeThresholdSetter() {
    hide();
    cvReleaseImage(&origImg);
    cvReleaseImage(&edgeImg);
}

/**
    Sets edge detector thresholds, creates edge image, and displays 
    the output window.
    @param lowThresh low threshold for edge detector
    @param highThresh high threshold for edge detector
 */
void EdgeThresholdSetter::setThresholds(int lowThresh, int highThresh) {
    this->highThresh = highThresh;
    this->lowThresh = lowThresh;

    if (edgeImg == NULL) {
        edgeImg = ImgFun::newImage(cvSize(origImg->width, origImg->height),
                                   IPL_DEPTH_8U, 1);
    }  

    cvCanny(origImg, edgeImg, lowThresh, highThresh, APERTURE_SIZE);
    
    display();
}

/**
    Returns the current edge detector thresholds.
    @param lowThresh low threshold is returned here
    @param highThresh high threshold is returned here
 */
void EdgeThresholdSetter::getThresholds(int &lowThresh, int &highThresh) {
    lowThresh = this->lowThresh;
    highThresh = this->highThresh;
}

/**
    Hides the output window.
 */
void EdgeThresholdSetter::hide() {
    cvDestroyWindow(ETS_WINDOW_NAME);
}

/**
    Opens the output window, which shows the edge image.
 */
void EdgeThresholdSetter::display() {
    cvNamedWindow(ETS_WINDOW_NAME, CV_WINDOW_AUTOSIZE);
    cvShowImage(ETS_WINDOW_NAME, edgeImg);
}
